@@ -490,8 +490,6 @@
 #define CACHEFILE ".snmp-exec-cache"
 #define MAXCACHESIZE (200*80)   /* roughly 200 lines max */
 
-#define MAXDISKS 50                      /* can't scan more than this number */
-
 /* misc defaults */
 
 /* default of 100 meg minimum if the minimum size is not specified in
@@ -656,12 +654,22 @@
     available.  */
 #undef SNMP_TRANSPORT_TCPIPV6_DOMAIN
 
+/*  This is defined if support for the UDP/IP transport domain is
+    available.   */
+#undef SNMP_TRANSPORT_TLS_DOMAIN
+
+/*  This is defined if support for stdin/out transport domain is available.   */
+#undef SNMP_TRANSPORT_STD_DOMAIN
+
 /* define this if the USM security module is available */
 #undef SNMP_SECMOD_USM
 
 /* define this if the KSM (kerberos based snmp) security module is available */
 #undef SNMP_SECMOD_KSM
 
+/* define this if the local security module is available */
+#undef SNMP_SECMOD_LOCALSM
+
 /* define this if we're using the new MIT crypto API */
 #undef MIT_NEW_CRYPTO
 
