@@ -84,10 +84,6 @@
 #include <inet/mib2.h>
 #endif
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -220,18 +216,20 @@ init_agent_read_config(const char *app)
 
     if (netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, 
 			       NETSNMP_DS_AGENT_ROLE) == MASTER_AGENT) {
-#if !defined(DISABLE_SNMPV1) || !defined(DISABLE_SNMPV2C)
+#ifndef DISABLE_SNMPV1
         register_app_config_handler("trapsink",
                                     snmpd_parse_config_trapsink,
                                     snmpd_free_trapsinks,
                                     "host [community] [port]");
+#endif
+#ifndef DISABLE_SNMPV2C
         register_app_config_handler("trap2sink",
                                     snmpd_parse_config_trap2sink, NULL,
                                     "host [community] [port]");
         register_app_config_handler("informsink",
                                     snmpd_parse_config_informsink, NULL,
                                     "host [community] [port]");
-#endif /* support for community based SNMP */
+#endif
         register_app_config_handler("trapsess",
                                     snmpd_parse_config_trapsess, NULL,
                                     "[snmpcmdargs] host");
@@ -272,6 +270,7 @@ update_config(void)
     snmp_call_callbacks(SNMP_CALLBACK_APPLICATION,
                         SNMPD_CALLBACK_PRE_UPDATE_CONFIG, NULL);
     free_config();
+    vacm_standard_views(0,0,NULL,NULL);
     read_configs();
 }
 
