@@ -51,10 +51,6 @@
 #include <netinet/in.h>
 #endif
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/agent/agent_callbacks.h>
@@ -208,7 +204,29 @@ netsnmp_subtree_replace_first(netsnmp_subtree *new_tree,
     return add_subtree(new_tree, context_name);
 }
 
+NETSNMP_INLINE void
+netsnmp_subtree_change_next(netsnmp_subtree *ptr, netsnmp_subtree *thenext)
+{
+    ptr->next = thenext;
+    if (thenext)
+        netsnmp_oid_compare_ll(ptr->start_a,
+                               ptr->start_len,
+                               thenext->start_a,
+                               thenext->start_len,
+                               &thenext->oid_off);
+}
 
+NETSNMP_INLINE void
+netsnmp_subtree_change_prev(netsnmp_subtree *ptr, netsnmp_subtree *theprev)
+{
+    ptr->prev = theprev;
+    if (theprev)
+        netsnmp_oid_compare_ll(theprev->start_a,
+                               theprev->start_len,
+                               ptr->start_a,
+                               ptr->start_len,
+                               &ptr->oid_off);
+}
 
 int
 netsnmp_subtree_compare(const netsnmp_subtree *ap, const netsnmp_subtree *bp)
@@ -239,10 +257,10 @@ netsnmp_subtree_join(netsnmp_subtree *root)
 	    s->end_a      = NULL;
 
             for (c = root; c != NULL; c = c->children) {
-                c->next = s->next;
+                netsnmp_subtree_change_next(c, s->next);
             }
             for (c = s; c != NULL; c = c->children) {
-                c->prev = root;
+                netsnmp_subtree_change_prev(c, root);
             }
             DEBUGMSG(("subtree", " so new end "));
             DEBUGMSGOID(("subtree", root->end_a, root->end_len));
@@ -360,13 +378,13 @@ netsnmp_subtree_split(netsnmp_subtree *current, oid name[], int name_len)
 
     /* Retain the correct linking of the list */
     for (ptr = current; ptr != NULL; ptr = ptr->children) {
-      ptr->next = new_sub;
+        netsnmp_subtree_change_next(ptr, new_sub);
     }
     for (ptr = new_sub; ptr != NULL; ptr = ptr->children) {
-      ptr->prev = current;
+        netsnmp_subtree_change_prev(ptr, current);
     }
     for (ptr = new_sub->next; ptr != NULL; ptr=ptr->children) {
-      ptr->prev = new_sub;
+        netsnmp_subtree_change_prev(ptr, new_sub);
     }
 
     return new_sub;
@@ -431,19 +449,20 @@ netsnmp_subtree_load(netsnmp_subtree *new_sub, const char *context_name)
 	    existing registrations.  */
 
 	if (tree2) {
-	    new_sub->prev = tree2->prev;
-	    tree2->prev   = new_sub;
+            netsnmp_subtree_change_prev(new_sub, tree2->prev);
+            netsnmp_subtree_change_prev(tree2, new_sub);
 	} else {
-	    new_sub->prev = netsnmp_subtree_find_prev(new_sub->start_a,
-				      new_sub->start_len, NULL, context_name);
+            netsnmp_subtree_change_prev(new_sub,
+                                        netsnmp_subtree_find_prev(new_sub->start_a,
+                                                                  new_sub->start_len, NULL, context_name));
 
 	    if (new_sub->prev) {
-		new_sub->prev->next = new_sub;
+                netsnmp_subtree_change_next(new_sub->prev, new_sub);
 	    } else {
 		netsnmp_subtree_replace_first(new_sub, context_name);
 	    }
 
-	    new_sub->next = tree2;
+            netsnmp_subtree_change_next(new_sub, tree2);
 
 	    /* If there was any overlap, recurse to merge in the overlapping
 	       region (including anything that may follow the overlap).  */
@@ -516,19 +535,19 @@ netsnmp_subtree_load(netsnmp_subtree *new_sub, const char *context_name)
 	    if (prev) {
 		prev->children    = new_sub;
 		new_sub->children = next;
-		new_sub->prev = prev->prev;
-		new_sub->next = prev->next;
+                netsnmp_subtree_change_prev(new_sub, prev->prev);
+                netsnmp_subtree_change_next(new_sub, prev->next);
 	    } else {
 		new_sub->children = next;
-		new_sub->prev = next->prev;
-		new_sub->next = next->next;
+                netsnmp_subtree_change_prev(new_sub, next->prev);
+                netsnmp_subtree_change_next(new_sub, next->next);
 	
 		for (next = new_sub->next; next != NULL;next = next->children){
-		    next->prev = new_sub;
+                    netsnmp_subtree_change_prev(next, new_sub);
 		}
 
 		for (prev = new_sub->prev; prev != NULL;prev = prev->children){
-		    prev->next = new_sub;
+                    netsnmp_subtree_change_next(prev, new_sub);
 		}
 	    }
 	    break;
@@ -588,7 +607,7 @@ netsnmp_register_mib(const char *moduleName,
     DEBUGMSGOIDRANGE(("register_mib", mibloc, mibloclen, range_subid,
                       range_ubound));
     DEBUGMSG(("register_mib", " with context \"%s\"\n",
-                                    (context ? context : "(null)")));
+              SNMP_STRORNULL(context)));
 
     /*
      * verify that the passed context is equal to the context
@@ -883,10 +902,12 @@ netsnmp_subtree_unload(netsnmp_subtree *sub, netsnmp_subtree *prev, const char *
      */
 
     if (sub->children == NULL) {        /* just remove this node completely */
-        for (ptr = sub->prev; ptr; ptr = ptr->children)
-            ptr->next = sub->next;
-        for (ptr = sub->next; ptr; ptr = ptr->children)
-            ptr->prev = sub->prev;
+        for (ptr = sub->prev; ptr; ptr = ptr->children) {
+            netsnmp_subtree_change_next(ptr, sub->next);
+        }
+        for (ptr = sub->next; ptr; ptr = ptr->children) {
+            netsnmp_subtree_change_prev(ptr, sub->prev);
+        }
 
 	if (sub->prev == NULL) {
 	    netsnmp_subtree_replace_first(sub->next, context);
@@ -894,9 +915,9 @@ netsnmp_subtree_unload(netsnmp_subtree *sub, netsnmp_subtree *prev, const char *
 
     } else {
         for (ptr = sub->prev; ptr; ptr = ptr->children)
-            ptr->next = sub->children;
+            netsnmp_subtree_change_next(ptr, sub->children);
         for (ptr = sub->next; ptr; ptr = ptr->children)
-            ptr->prev = sub->children;
+            netsnmp_subtree_change_prev(ptr, sub->children);
 
 	if (sub->prev == NULL) {
 	    netsnmp_subtree_replace_first(sub->children, context);
@@ -1136,7 +1157,7 @@ unregister_mibs_by_session(netsnmp_session * ss)
                     (!(!ss || ss->flags & SNMP_FLAGS_SUBSESSION) && child->session &&
                      child->session->subsession == ss)) {
 
-                    memset(&rp, 0x0, sizeof(rp));
+                    memset(&rp,0x0,sizeof(rp));
                     rp.name = child->name_a;
 		    child->name_a = NULL;
                     rp.namelen = child->namelen;
@@ -1437,6 +1458,7 @@ netsnmp_subtree_find_prev(oid *name, size_t len, netsnmp_subtree *subtree,
     lookup_cache *lookup_cache = NULL;
     netsnmp_subtree *myptr = NULL, *previous = NULL;
     int cmp = 1;
+    int ll_off = 0;
 
     if (subtree) {
         myptr = subtree;
@@ -1455,8 +1477,38 @@ netsnmp_subtree_find_prev(oid *name, size_t len, netsnmp_subtree *subtree,
         }
     }
 
+    /*
+     * this optimization causes a segfault on sf cf alpha-linux1.
+     * ifdef out until someone figures out why and fixes it. xxx-rks 20051117
+     */
+#ifndef __alpha
+#define WTEST_OPTIMIZATION 1
+#endif
+#ifdef WTEST_OPTIMIZATION
+    DEBUGMSGTL(("wtest","oid in: "));
+    DEBUGMSGOID(("wtest", name, len));
+    DEBUGMSG(("wtest","\n"));
+#endif
     for (; myptr != NULL; previous = myptr, myptr = myptr->next) {
+#ifdef WTEST_OPTIMIZATION
+        /* Compare the incoming oid with the linked list.  If we have
+           results of previous compares, its faster to make sure the
+           length we differed in the last check is greater than the
+           length between this pointer and the last then we don't need
+           to actually perform a comparison */
+        DEBUGMSGTL(("wtest","oid cmp: "));
+        DEBUGMSGOID(("wtest", myptr->start_a, myptr->start_len));
+        DEBUGMSG(("wtest","  --- off = %d, in off = %d test = %d\n",
+                  myptr->oid_off, ll_off,
+                  !(ll_off && myptr->oid_off &&
+                    myptr->oid_off > ll_off)));
+        if (!(ll_off && myptr->oid_off && myptr->oid_off > ll_off) &&
+            netsnmp_oid_compare_ll(name, len,
+                                   myptr->start_a, myptr->start_len,
+                                   &ll_off) < 0) {
+#else
         if (snmp_oid_compare(name, len, myptr->start_a, myptr->start_len) < 0) {
+#endif
             if (lookup_cache_size && previous && cmp) {
                 if (lookup_cache) {
                     lookup_cache_replace(lookup_cache, myptr, previous);
@@ -1732,128 +1784,6 @@ dump_registry(void)
     dump_idx_registry();
 }
 
-
-int	external_readfd[NUM_EXTERNAL_FDS],   external_readfdlen   = 0;
-int	external_writefd[NUM_EXTERNAL_FDS],  external_writefdlen  = 0;
-int	external_exceptfd[NUM_EXTERNAL_FDS], external_exceptfdlen = 0;
-void  (*external_readfdfunc[NUM_EXTERNAL_FDS]) (int, void *);
-void  (*external_writefdfunc[NUM_EXTERNAL_FDS]) (int, void *);
-void  (*external_exceptfdfunc[NUM_EXTERNAL_FDS]) (int, void *);
-void   *external_readfd_data[NUM_EXTERNAL_FDS];
-void   *external_writefd_data[NUM_EXTERNAL_FDS];
-void   *external_exceptfd_data[NUM_EXTERNAL_FDS];
-
-int
-register_readfd(int fd, void (*func) (int, void *), void *data)
-{
-    if (external_readfdlen < NUM_EXTERNAL_FDS) {
-        external_readfd[external_readfdlen] = fd;
-        external_readfdfunc[external_readfdlen] = func;
-        external_readfd_data[external_readfdlen] = data;
-        external_readfdlen++;
-        DEBUGMSGTL(("register_readfd", "registered fd %d\n", fd));
-        return FD_REGISTERED_OK;
-    } else {
-        snmp_log(LOG_CRIT, "register_readfd: too many file descriptors\n");
-        return FD_REGISTRATION_FAILED;
-    }
-}
-
-int
-register_writefd(int fd, void (*func) (int, void *), void *data)
-{
-    if (external_writefdlen < NUM_EXTERNAL_FDS) {
-        external_writefd[external_writefdlen] = fd;
-        external_writefdfunc[external_writefdlen] = func;
-        external_writefd_data[external_writefdlen] = data;
-        external_writefdlen++;
-        DEBUGMSGTL(("register_writefd", "registered fd %d\n", fd));
-        return FD_REGISTERED_OK;
-    } else {
-        snmp_log(LOG_CRIT,
-                 "register_writefd: too many file descriptors\n");
-        return FD_REGISTRATION_FAILED;
-    }
-}
-
-int
-register_exceptfd(int fd, void (*func) (int, void *), void *data)
-{
-    if (external_exceptfdlen < NUM_EXTERNAL_FDS) {
-        external_exceptfd[external_exceptfdlen] = fd;
-        external_exceptfdfunc[external_exceptfdlen] = func;
-        external_exceptfd_data[external_exceptfdlen] = data;
-        external_exceptfdlen++;
-        DEBUGMSGTL(("register_exceptfd", "registered fd %d\n", fd));
-        return FD_REGISTERED_OK;
-    } else {
-        snmp_log(LOG_CRIT,
-                 "register_exceptfd: too many file descriptors\n");
-        return FD_REGISTRATION_FAILED;
-    }
-}
-
-int
-unregister_readfd(int fd)
-{
-    int             i, j;
-
-    for (i = 0; i < external_readfdlen; i++) {
-        if (external_readfd[i] == fd) {
-            external_readfdlen--;
-            for (j = i; j < external_readfdlen; j++) {
-                external_readfd[j] = external_readfd[j + 1];
-                external_readfdfunc[j] = external_readfdfunc[j + 1];
-                external_readfd_data[j] = external_readfd_data[j + 1];
-            }
-            DEBUGMSGTL(("unregister_readfd", "unregistered fd %d\n", fd));
-            return FD_UNREGISTERED_OK;
-        }
-    }
-    return FD_NO_SUCH_REGISTRATION;
-}
-
-int
-unregister_writefd(int fd)
-{
-    int             i, j;
-
-    for (i = 0; i < external_writefdlen; i++) {
-        if (external_writefd[i] == fd) {
-            external_writefdlen--;
-            for (j = i; j < external_writefdlen; j++) {
-                external_writefd[j] = external_writefd[j + 1];
-                external_writefdfunc[j] = external_writefdfunc[j + 1];
-                external_writefd_data[j] = external_writefd_data[j + 1];
-            }
-            DEBUGMSGTL(("unregister_writefd", "unregistered fd %d\n", fd));
-            return FD_UNREGISTERED_OK;
-        }
-    }
-    return FD_NO_SUCH_REGISTRATION;
-}
-
-int
-unregister_exceptfd(int fd)
-{
-    int             i, j;
-
-    for (i = 0; i < external_exceptfdlen; i++) {
-        if (external_exceptfd[i] == fd) {
-            external_exceptfdlen--;
-            for (j = i; j < external_exceptfdlen; j++) {
-                external_exceptfd[j] = external_exceptfd[j + 1];
-                external_exceptfdfunc[j] = external_exceptfdfunc[j + 1];
-                external_exceptfd_data[j] = external_exceptfd_data[j + 1];
-            }
-            DEBUGMSGTL(("unregister_exceptfd", "unregistered fd %d\n",
-                        fd));
-            return FD_UNREGISTERED_OK;
-        }
-    }
-    return FD_NO_SUCH_REGISTRATION;
-}
-
 int             external_signal_scheduled[NUM_EXTERNAL_SIGS];
 void            (*external_signal_handler[NUM_EXTERNAL_SIGS]) (int);
 
