@@ -1,6 +1,6 @@
 /*
  * table_container.c
- * $Id: table_container.c,v 1.13.2.5 2006/01/27 09:05:31 tanders Exp $
+ * $Id: table_container.c,v 1.21.2.2 2006/01/25 16:26:38 dts12 Exp $
  */
 
 #include <net-snmp/net-snmp-config.h>
@@ -19,10 +19,6 @@
 #include <net-snmp/library/container.h>
 #include <net-snmp/library/snmp_assert.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 /*
  * snmp.h:#define SNMP_MSG_INTERNAL_SET_BEGIN        -1 
  * snmp.h:#define SNMP_MSG_INTERNAL_SET_RESERVE1     0 
@@ -142,6 +138,11 @@ _container_table_handler(netsnmp_mib_handler *handler,
                          netsnmp_agent_request_info *agtreq_info,
                          netsnmp_request_info *requests);
 
+static void *
+_find_next_row(netsnmp_container *c,
+               netsnmp_table_request_info *tblreq,
+               void * key);
+
 /**********************************************************************
  **********************************************************************
  *                                                                    *
@@ -151,8 +152,104 @@ _container_table_handler(netsnmp_mib_handler *handler,
  *                                                                    *
  **********************************************************************
  **********************************************************************/
-/** register specified callbacks for the specified table/oid.
-*/
+
+/* ==================================
+ *
+ * Container Table API: Table maintenance
+ *
+ * ================================== */
+
+container_table_data *
+netsnmp_tcontainer_create_table( const char *name,
+                                 netsnmp_container *container, long flags )
+{
+    container_table_data *table;
+
+    table = SNMP_MALLOC_TYPEDEF(container_table_data);
+    if (!table)
+        return NULL;
+    if (container)
+        table->table = container;
+    else {
+        table->table = netsnmp_container_find("table_container");
+        if (!table->table) {
+            SNMP_FREE(table);
+            return NULL;
+        }
+    }
+
+    if (flags)
+        table->key_type = flags & 0x03;  /* Use lowest two bits */
+    else
+        table->key_type = TABLE_CONTAINER_KEY_NETSNMP_INDEX;
+
+    if (!table->table->compare)
+         table->table->compare  = netsnmp_compare_netsnmp_index;
+    if (!table->table->ncompare)
+         table->table->ncompare = netsnmp_ncompare_netsnmp_index;
+
+    return table;
+}
+
+void
+netsnmp_tcontainer_delete_table( container_table_data *table )
+{
+    if (!table)
+       return;
+
+    if (table->table)
+       CONTAINER_FREE(table->table);
+    
+    SNMP_FREE(table);
+    return;
+}
+
+    /*
+     * The various standalone row operation routines
+     *    (create/clone/copy/delete)
+     * will be specific to a particular table,
+     *    so can't be implemented here.
+     */
+
+int
+netsnmp_tcontainer_add_row( container_table_data *table, netsnmp_index *row )
+{
+    if (!table || !table->table || !row)
+        return -1;
+    CONTAINER_INSERT( table->table, row );
+    return 0;
+}
+
+netsnmp_index *
+netsnmp_tcontainer_remove_row( container_table_data *table, netsnmp_index *row )
+{
+    if (!table || !table->table || !row)
+        return NULL;
+    CONTAINER_REMOVE( table->table, row );
+    return NULL;
+}
+
+int
+netsnmp_tcontainer_replace_row( container_table_data *table,
+                                netsnmp_index *old_row, netsnmp_index *new_row )
+{
+    if (!table || !table->table || !old_row || !new_row)
+        return -1;
+    netsnmp_tcontainer_remove_row( table, old_row );
+    netsnmp_tcontainer_add_row(    table, new_row );
+    return 0;
+}
+
+    /* netsnmp_tcontainer_remove_delete_row() will be table-specific too */
+
+
+/* ==================================
+ *
+ * Container Table API: MIB maintenance
+ *
+ * ================================== */
+
+/** returns a netsnmp_mib_handler object for the table_container helper */
 netsnmp_mib_handler *
 netsnmp_container_table_handler_get(netsnmp_table_registration_info *tabreg,
                                     netsnmp_container *container, char key_type)
@@ -218,7 +315,104 @@ netsnmp_container_table_register(netsnmp_handler_registration *reginfo,
     return netsnmp_register_table(reginfo, tabreg);
 }
 
-/** @} */
+/** retrieve the container used by the table_container helper */
+netsnmp_container*
+netsnmp_container_table_container_extract(netsnmp_request_info *request)
+{
+    return (netsnmp_container *)
+         netsnmp_request_get_list_data(request, TABLE_CONTAINER_CONTAINER);
+}
+
+#ifndef NETSNMP_USE_INLINE
+/** find the context data used by the table_container helper */
+void *
+netsnmp_container_table_row_extract(netsnmp_request_info *request)
+{
+    /*
+     * NOTE: this function must match in table_container.c and table_container.h.
+     *       if you change one, change them both!
+     */
+    return netsnmp_request_get_list_data(request, TABLE_CONTAINER_ROW);
+}
+/** find the context data used by the table_container helper */
+void *
+netsnmp_container_table_extract_context(netsnmp_request_info *request)
+{
+    /*
+     * NOTE: this function must match in table_container.c and table_container.h.
+     *       if you change one, change them both!
+     */
+    return netsnmp_request_get_list_data(request, TABLE_CONTAINER_ROW);
+}
+#endif /* inline */
+
+/** inserts a newly created table_container entry into a request list */
+void
+netsnmp_container_table_row_insert(netsnmp_request_info *request,
+                                   netsnmp_index        *row)
+{
+    netsnmp_request_info       *req;
+    netsnmp_table_request_info *table_info = NULL;
+    netsnmp_variable_list      *this_index = NULL;
+    netsnmp_variable_list      *that_index = NULL;
+    oid      base_oid[] = {0, 0};	/* Make sure index OIDs are legal! */
+    oid      this_oid[MAX_OID_LEN];
+    oid      that_oid[MAX_OID_LEN];
+    size_t   this_oid_len, that_oid_len;
+
+    if (!request)
+        return;
+
+    /*
+     * We'll add the new row information to any request
+     * structure with the same index values as the request
+     * passed in (which includes that one!).
+     *
+     * So construct an OID based on these index values.
+     */
+
+    table_info = netsnmp_extract_table_info(request);
+    this_index = table_info->indexes;
+    build_oid_noalloc(this_oid, MAX_OID_LEN, &this_oid_len,
+                      base_oid, 2, this_index);
+
+    /*
+     * We need to look through the whole of the request list
+     * (as received by the current handler), as there's no
+     * guarantee that this routine will be called by the first
+     * varbind that refers to this row.
+     *   In particular, a RowStatus controlled row creation
+     * may easily occur later in the variable list.
+     *
+     * So first, we rewind to the head of the list....
+     */
+    for (req=request; req->prev; req=req->prev)
+        ;
+
+    /*
+     * ... and then start looking for matching indexes
+     * (by constructing OIDs from these index values)
+     */
+    for (; req; req=req->next) {
+        if (req->processed) 
+            continue;
+        
+        table_info = netsnmp_extract_table_info(req);
+        that_index = table_info->indexes;
+        build_oid_noalloc(that_oid, MAX_OID_LEN, &that_oid_len,
+                          base_oid, 2, that_index);
+      
+        /*
+         * This request has the same index values,
+         * so add the newly-created row information.
+         */
+        if (snmp_oid_compare(this_oid, this_oid_len,
+                             that_oid, that_oid_len) == 0) {
+            netsnmp_request_add_list_data(req,
+                netsnmp_create_data_list(TABLE_CONTAINER_ROW, row, NULL));
+        }
+    }
+}
 
 /** @cond */
 /**********************************************************************
@@ -252,75 +446,6 @@ _set_key( container_table_data * tad, netsnmp_request_info *request,
         *key = NULL;
 }
 
-static void *
-_find_next_row(netsnmp_container *c,
-               netsnmp_table_request_info *tblreq,
-               void * key)
-{
-    void *row = NULL;
-
-    if (!c || !tblreq || !tblreq->reg_info ) {
-        snmp_log(LOG_ERR,"_find_next_row param error\n");
-        return NULL;
-    }
-
-    /*
-     * table helper should have made sure we aren't below our minimum column
-     */
-    netsnmp_assert(tblreq->colnum >= tblreq->reg_info->min_column);
-
-    /*
-     * if no indexes then use first row.
-     */
-    if(tblreq->number_indexes == 0) {
-        row = CONTAINER_FIRST(c);
-    } else {
-
-        if(NULL == key) {
-            netsnmp_index index;
-            index.oids = tblreq->index_oid;
-            index.len = tblreq->index_oid_len;
-            row = CONTAINER_NEXT(c, &index);
-        }
-        else
-            row = CONTAINER_NEXT(c, key);
-
-        /*
-         * we don't have a row, but we might be at the end of a
-         * column, so try the next column.
-         */
-        if (NULL == row) {
-            /*
-             * don't set tblreq next_col unless we know there is one,
-             * so we don't mess up table handler sparse table processing.
-             */
-            oid next_col = netsnmp_table_next_column(tblreq);
-            if (0 != next_col) {
-                tblreq->colnum = next_col;
-                row = CONTAINER_FIRST(c);
-            }
-        }
-    }
-    
-    return row;
-}
-
-/**
- * deprecated, backwards compatability only
- *
- * expected impact to remove: none
- *  - used between helpers, shouldn't have been used by end users
- *
- * replacement: none
- *  - never should have been a public method in the first place
- */
-netsnmp_index *
-netsnmp_table_index_find_next_row(netsnmp_container *c,
-                                  netsnmp_table_request_info *tblreq)
-{
-    return _find_next_row(c, tblreq, NULL );
-}
-
 
 NETSNMP_STATIC_INLINE void
 _data_lookup(netsnmp_handler_registration *reginfo,
@@ -370,6 +495,8 @@ _data_lookup(netsnmp_handler_registration *reginfo,
                 netsnmp_update_variable_list_from_index(tblreq_info);
             }
             else if (TABLE_CONTAINER_KEY_VARBIND_INDEX == tad->key_type) {
+                /** xxx-rks: shouldn't tblreq_info->indexes be updated
+                    before we call this?? */
                 netsnmp_update_indexes_from_variable_list(tblreq_info);
             }
 
@@ -408,7 +535,7 @@ _data_lookup(netsnmp_handler_registration *reginfo,
     /*
      * save the data and table in the request.
      */
-    if (SNMP_ENDOFMIBVIEW != request->status) {
+    if (SNMP_ENDOFMIBVIEW != request->requestvb->type) {
         if (NULL != row)
             netsnmp_request_add_list_data(request,
                                           netsnmp_create_data_list
@@ -521,90 +648,86 @@ _container_table_handler(netsnmp_mib_handler *handler,
 }
 /** @endcond */
 
-/** retrieve the container used by the table_container helper */
-netsnmp_container*
-netsnmp_container_table_container_extract(netsnmp_request_info *request)
+
+/* ==================================
+ *
+ * Container Table API: Row operations
+ *
+ * ================================== */
+
+static void *
+_find_next_row(netsnmp_container *c,
+               netsnmp_table_request_info *tblreq,
+               void * key)
 {
-    return (netsnmp_container *)
-         netsnmp_request_get_list_data(request, TABLE_CONTAINER_CONTAINER);
-}
+    void *row = NULL;
 
-/** inserts a newly created table_container entry into a request list */
-void
-netsnmp_container_table_row_insert(netsnmp_request_info *request,
-                                   netsnmp_index        *row)
-{
-    netsnmp_request_info       *req;
-    netsnmp_table_request_info *table_info = NULL;
-    netsnmp_variable_list      *this_index = NULL;
-    netsnmp_variable_list      *that_index = NULL;
-    oid      base_oid[] = {0, 0};	/* Make sure index OIDs are legal! */
-    oid      this_oid[MAX_OID_LEN];
-    oid      that_oid[MAX_OID_LEN];
-    size_t   this_oid_len, that_oid_len;
-
-    if (!request)
-        return;
+    if (!c || !tblreq || !tblreq->reg_info ) {
+        snmp_log(LOG_ERR,"_find_next_row param error\n");
+        return NULL;
+    }
 
     /*
-     * We'll add the new row information to any request
-     * structure with the same index values as the request
-     * passed in (which includes that one!).
-     *
-     * So construct an OID based on these index values.
+     * table helper should have made sure we aren't below our minimum column
      */
-
-    table_info = netsnmp_extract_table_info(request);
-    this_index = table_info->indexes;
-    build_oid_noalloc(this_oid, MAX_OID_LEN, &this_oid_len,
-                      base_oid, 2, this_index);
+    netsnmp_assert(tblreq->colnum >= tblreq->reg_info->min_column);
 
     /*
-     * We need to look through the whole of the request list
-     * (as received by the current handler), as there's no
-     * guarantee that this routine will be called by the first
-     * varbind that refers to this row.
-     *   In particular, a RowStatus controlled row creation
-     * may easily occur later in the variable list.
-     *
-     * So first, we rewind to the head of the list....
+     * if no indexes then use first row.
      */
-    for (req=request; req->prev; req=req->prev)
-        ;
+    if(tblreq->number_indexes == 0) {
+        row = CONTAINER_FIRST(c);
+    } else {
+
+        if(NULL == key) {
+            netsnmp_index index;
+            index.oids = tblreq->index_oid;
+            index.len = tblreq->index_oid_len;
+            row = CONTAINER_NEXT(c, &index);
+        }
+        else
+            row = CONTAINER_NEXT(c, key);
 
-    /*
-     * ... and then start looking for matching indexes
-     * (by constructing OIDs from these index values)
-     */
-    for (; req; req=req->next) {
-        if (req->processed)
-            continue;
-        table_info = netsnmp_extract_table_info(req);
-        that_index = table_info->indexes;
-        build_oid_noalloc(that_oid, MAX_OID_LEN, &that_oid_len,
-                          base_oid, 2, that_index);
-      
         /*
-         * This request has the same index values,
-         * so add the newly-created row information.
+         * we don't have a row, but we might be at the end of a
+         * column, so try the next column.
          */
-        if (snmp_oid_compare(this_oid, this_oid_len,
-                             that_oid, that_oid_len) == 0) {
-            netsnmp_request_add_list_data(req,
-                netsnmp_create_data_list(TABLE_CONTAINER_ROW, row, NULL));
+        if (NULL == row) {
+            /*
+             * don't set tblreq next_col unless we know there is one,
+             * so we don't mess up table handler sparse table processing.
+             */
+            oid next_col = netsnmp_table_next_column(tblreq);
+            if (0 != next_col) {
+                tblreq->colnum = next_col;
+                row = CONTAINER_FIRST(c);
+            }
         }
     }
+    
+    return row;
 }
 
-#ifndef NETSNMP_USE_INLINE
-/** find the context data used by the table_container helper */
-void *
-netsnmp_container_table_row_extract(netsnmp_request_info *request)
+/**
+ * deprecated, backwards compatability only
+ *
+ * expected impact to remove: none
+ *  - used between helpers, shouldn't have been used by end users
+ *
+ * replacement: none
+ *  - never should have been a public method in the first place
+ */
+netsnmp_index *
+netsnmp_table_index_find_next_row(netsnmp_container *c,
+                                  netsnmp_table_request_info *tblreq)
 {
-    /*
-     * NOTE: this function must match in table_container.c and table_container.h.
-     *       if you change one, change them both!
-     */
-    return netsnmp_request_get_list_data(request, TABLE_CONTAINER_ROW);
+    return _find_next_row(c, tblreq, NULL );
 }
-#endif /* inline */
+
+/* ==================================
+ *
+ * Container Table API: Index operations
+ *
+ * ================================== */
+
+/** @} */
