@@ -98,9 +98,71 @@
 #include <net-snmp/agent/table_iterator.h>
 #include <net-snmp/agent/stash_cache.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
+/* ==================================
+ *
+ * Iterator API: Table maintenance
+ *
+ * ================================== */
+
+    /*
+     * Iterator-based tables are typically maintained by external
+     *  code, and this helper is really only concerned with
+     *  mapping between a walk through this local representation,
+     *  and the requirements of SNMP table ordering.
+     * However, there's a case to be made for considering the
+     *  iterator info structure as encapsulating the table, so
+     *  it's probably worth defining the table creation/deletion
+     *  routines from the generic API.
+     *
+     * Time will show whether this is a sensible approach or not.
+     */
+netsnmp_iterator_info *
+netsnmp_iterator_create_table( Netsnmp_First_Data_Point *firstDP,
+                               Netsnmp_Next_Data_Point  *nextDP,
+                               Netsnmp_First_Data_Point *getidx,
+                               netsnmp_variable_list    *indexes)
+{
+    netsnmp_iterator_info *iinfo =
+        SNMP_MALLOC_TYPEDEF(netsnmp_iterator_info);
+
+    if ( !iinfo )
+        return NULL;
+
+    if ( indexes )
+        iinfo->indexes = snmp_clone_varbind(indexes);
+    iinfo->get_first_data_point = firstDP;
+    iinfo->get_next_data_point  = nextDP;
+    iinfo->get_row_indexes      = getidx;
+
+    return iinfo;
+}
+
+void
+netsnmp_iterator_delete_table( netsnmp_iterator_info *iinfo )
+{
+    if (!iinfo)
+        return;
+
+    if (iinfo->indexes) {
+        snmp_free_varbind( iinfo->indexes );
+        iinfo->indexes = NULL;
+    }
+    SNMP_FREE( iinfo );
+}
+
+    /*
+     * The rest of the table maintenance section of the
+     *   generic table API is Not Applicable to this helper.
+     *
+     * The contents of a iterator-based table will be
+     *  maintained by the table-specific module itself.
+     */
+
+/* ==================================
+ *
+ * Iterator API: MIB maintenance
+ *
+ * ================================== */
 
 /** returns a netsnmp_mib_handler object for the table_iterator helper */
 netsnmp_mib_handler *
@@ -122,7 +184,6 @@ netsnmp_get_table_iterator_handler(netsnmp_iterator_info *iinfo)
     return me;
 }
 
-
 /** 
  * Creates and registers a table iterator helper handler calling 
  * netsnmp_create_handler with a handler name set to TABLE_ITERATOR_NAME 
@@ -144,10 +205,14 @@ int
 netsnmp_register_table_iterator(netsnmp_handler_registration *reginfo,
                                 netsnmp_iterator_info *iinfo)
 {
+    reginfo->modes |= HANDLER_CAN_STASH;
     netsnmp_inject_handler(reginfo,
                            netsnmp_get_table_iterator_handler(iinfo));
     if (!iinfo)
         return SNMPERR_GENERR;
+    if (!iinfo->indexes && iinfo->table_reginfo &&
+                           iinfo->table_reginfo->indexes )
+        iinfo->indexes = snmp_clone_varbind( iinfo->table_reginfo->indexes );
 
     return netsnmp_register_table(reginfo, iinfo->table_reginfo);
 }
@@ -310,14 +375,13 @@ netsnmp_iterator_remember(netsnmp_request_info *request,
 }    
 
 #define TABLE_ITERATOR_NOTAGAIN 255
-/** implements the table_iterator helper */
+/* implements the table_iterator helper */
 int
 netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
                                       netsnmp_handler_registration *reginfo,
                                       netsnmp_agent_request_info *reqinfo,
                                       netsnmp_request_info *requests)
 {
-
     netsnmp_table_registration_info *tbl_info;
     netsnmp_table_request_info *table_info = NULL;
     oid             coloid[MAX_OID_LEN];
@@ -456,6 +520,10 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
                 }
             }
 
+            /* if sorted, pass in a hint */
+            if (iinfo->flags & NETSNMP_ITERATOR_FLAG_SORTED) {
+                callback_loop_context = table_info;
+            }
             index_search =
                 (iinfo->get_first_data_point) (&callback_loop_context,
                                                &callback_data_context,
@@ -726,4 +794,280 @@ netsnmp_table_iterator_helper_handler(netsnmp_mib_handler *handler,
     return SNMP_ERR_NOERROR;
 }
 
+/* ==================================
+ *
+ * Iterator API: Row operations
+ *
+ * ================================== */
+
+void *
+netsnmp_iterator_row_first( netsnmp_iterator_info *iinfo ) {
+    netsnmp_variable_list *vp1, *vp2;
+    void *ctx1, *ctx2;
+
+    if (!iinfo)
+        return NULL;
+
+    vp1 = snmp_clone_varbind(iinfo->indexes);
+    vp2 = iinfo->get_first_data_point( &ctx1, &ctx2, vp1, iinfo );
+
+    if (!vp2)
+        ctx2 = NULL;
+
+    /* free loop context ?? */
+    snmp_free_varbind( vp1 );
+    return ctx2;  /* or *ctx2 ?? */
+}
+
+void *
+netsnmp_iterator_row_get( netsnmp_iterator_info *iinfo, void *row )
+{
+    netsnmp_variable_list *vp1, *vp2;
+    void *ctx1, *ctx2;
+
+    if (!iinfo || !row)
+        return NULL;
+
+        /*
+         * This routine relies on being able to
+         *   determine the indexes for a given row.  
+         */
+    if (!iinfo->get_row_indexes)
+        return NULL;
+
+    vp1  = snmp_clone_varbind(iinfo->indexes);
+    ctx1 = row;   /* Probably only need one of these ... */
+    ctx2 = row;
+    vp2  = iinfo->get_row_indexes( &ctx1, &ctx2, vp1, iinfo );
+
+    ctx2 = NULL;
+    if (vp2) {
+        ctx2 = netsnmp_iterator_row_get_byidx( iinfo, vp2 );
+    }
+    snmp_free_varbind( vp1 );
+    return ctx2;
+}
+
+void *
+netsnmp_iterator_row_next( netsnmp_iterator_info *iinfo, void *row )
+{
+    netsnmp_variable_list *vp1, *vp2;
+    void *ctx1, *ctx2;
+
+    if (!iinfo || !row)
+        return NULL;
+
+        /*
+         * This routine relies on being able to
+         *   determine the indexes for a given row.  
+         */
+    if (!iinfo->get_row_indexes)
+        return NULL;
+
+    vp1  = snmp_clone_varbind(iinfo->indexes);
+    ctx1 = row;   /* Probably only need one of these ... */
+    ctx2 = row;
+    vp2  = iinfo->get_row_indexes( &ctx1, &ctx2, vp1, iinfo );
+
+    ctx2 = NULL;
+    if (vp2) {
+        ctx2 = netsnmp_iterator_row_next_byidx( iinfo, vp2 );
+    }
+    snmp_free_varbind( vp1 );
+    return ctx2;
+}
+
+void *
+netsnmp_iterator_row_get_byidx(  netsnmp_iterator_info *iinfo,
+                                 netsnmp_variable_list *indexes )
+{
+    oid    dummy[] = {0,0};   /* Keep 'build_oid' happy */
+    oid    instance[MAX_OID_LEN];
+    size_t len =    MAX_OID_LEN;
+
+    if (!iinfo || !indexes)
+        return NULL;
+
+    build_oid_noalloc(instance, MAX_OID_LEN, &len,
+                      dummy, 2, indexes);
+    return netsnmp_iterator_row_get_byoid( iinfo, instance+2, len-2 );
+}
+
+void *
+netsnmp_iterator_row_next_byidx( netsnmp_iterator_info *iinfo,
+                                 netsnmp_variable_list *indexes )
+{
+    oid    dummy[] = {0,0};
+    oid    instance[MAX_OID_LEN];
+    size_t len =    MAX_OID_LEN;
+
+    if (!iinfo || !indexes)
+        return NULL;
+
+    build_oid_noalloc(instance, MAX_OID_LEN, &len,
+                      dummy, 2, indexes);
+    return netsnmp_iterator_row_next_byoid( iinfo, instance+2, len-2 );
+}
+
+void *
+netsnmp_iterator_row_get_byoid(  netsnmp_iterator_info *iinfo,
+                                 oid *instance, size_t len )
+{
+    oid    dummy[] = {0,0};
+    oid    this_inst[ MAX_OID_LEN];
+    size_t this_len;
+    netsnmp_variable_list *vp1, *vp2;
+    void *ctx1, *ctx2;
+    int   n;
+
+    if (!iinfo || !iinfo->get_first_data_point
+               || !iinfo->get_next_data_point )
+        return NULL;
+
+    if ( !instance || !len )
+        return NULL;
+
+    vp1 = snmp_clone_varbind(iinfo->indexes);
+    vp2 = iinfo->get_first_data_point( &ctx1, &ctx2, vp1, iinfo );
+    DEBUGMSGTL(("table:iterator:get", "first DP: %x %x %x\n",
+                                       ctx1, ctx2, vp2));
+
+    /* XXX - free context ? */
+    
+    while ( vp2 ) {
+        this_len = MAX_OID_LEN;
+        build_oid_noalloc(this_inst, MAX_OID_LEN, &this_len, dummy, 2, vp2);
+        n = snmp_oid_compare( instance, len, this_inst+2, this_len-2 );
+        if ( n == 0 )
+            break;  /* Found matching row */
+
+        if (( n > 0) &&
+            (iinfo->flags & NETSNMP_ITERATOR_FLAG_SORTED)) {
+            vp2 = NULL;  /* Row not present */
+            break;
+        }
+        
+        vp2 = iinfo->get_next_data_point( &ctx1, &ctx2, vp2, iinfo );
+        DEBUGMSGTL(("table:iterator:get", "next DP: %x %x %x\n",
+                                           ctx1, ctx2, vp2));
+        /* XXX - free context ? */
+    }
+           
+    /* XXX - final free context ? */
+    snmp_free_varbind( vp1 );
+
+    return ( vp2 ? ctx2 : NULL );
+}
+
+void *
+netsnmp_iterator_row_next_byoid( netsnmp_iterator_info *iinfo,
+                                 oid *instance, size_t len )
+{
+    oid    dummy[] = {0,0};
+    oid    this_inst[ MAX_OID_LEN];
+    size_t this_len;
+    oid    best_inst[ MAX_OID_LEN];
+    size_t best_len = 0;
+    netsnmp_variable_list *vp1, *vp2;
+    void *ctx1, *ctx2;
+    int   n;
+
+    if (!iinfo || !iinfo->get_first_data_point
+               || !iinfo->get_next_data_point )
+        return NULL;
+
+    vp1 = snmp_clone_varbind(iinfo->indexes);
+    vp2 = iinfo->get_first_data_point( &ctx1, &ctx2, vp1, iinfo );
+    DEBUGMSGTL(("table:iterator:get", "first DP: %x %x %x\n",
+                                       ctx1, ctx2, vp2));
+
+    if ( !instance || !len ) {
+        snmp_free_varbind( vp1 );
+        return ( vp2 ? ctx2 : NULL );   /* First entry */
+    }
+
+    /* XXX - free context ? */
+    
+    while ( vp2 ) {
+        this_len = MAX_OID_LEN;
+        build_oid_noalloc(this_inst, MAX_OID_LEN, &this_len, dummy, 2, vp2);
+        n = snmp_oid_compare( instance, len, this_inst+2, this_len-2 );
+
+        /*
+         * Look for the best-fit candidate for the next row
+         *   (bearing in mind the rows may not be ordered "correctly")
+         */
+        if ( n > 0 ) {
+            if ( best_len == 0 ) {
+                memcpy( best_inst, this_inst, sizeof( this_inst ));
+                best_len = this_len;
+                if (iinfo->flags & NETSNMP_ITERATOR_FLAG_SORTED)
+                    break;
+            } else {
+                n = snmp_oid_compare( best_inst, best_len, this_inst, this_len );
+                if ( n < 0 ) {
+                    memcpy( best_inst, this_inst, sizeof( this_inst ));
+                    best_len = this_len;
+                    if (iinfo->flags & NETSNMP_ITERATOR_FLAG_SORTED)
+                        break;
+                }
+            }
+        }
+        
+        vp2 = iinfo->get_next_data_point( &ctx1, &ctx2, vp2, iinfo );
+        DEBUGMSGTL(("table:iterator:get", "next DP: %x %x %x\n",
+                                           ctx1, ctx2, vp2));
+        /* XXX - free context ? */
+    }
+           
+    /* XXX - final free context ? */
+    snmp_free_varbind( vp1 );
+
+    return ( vp2 ? ctx2 : NULL );
+}
+
+int
+netsnmp_iterator_row_count( netsnmp_iterator_info *iinfo )
+{
+    netsnmp_variable_list *vp1, *vp2;
+    void *ctx1, *ctx2;
+    int   i=0;
+
+    if (!iinfo || !iinfo->get_first_data_point
+               || !iinfo->get_next_data_point )
+        return 0;
+
+    vp1 = snmp_clone_varbind(iinfo->indexes);
+    vp2 = iinfo->get_first_data_point( &ctx1, &ctx2, vp1, iinfo );
+    if (!vp2) {
+        snmp_free_varbind( vp1 );
+        return 0;
+    }
+    
+    DEBUGMSGTL(("table:iterator:count", "first DP: %x %x %x\n",
+                                         ctx1, ctx2, vp2));
+
+    /* XXX - free context ? */
+
+    while (vp2) {
+        i++;
+        vp2 = iinfo->get_next_data_point( &ctx1, &ctx2, vp2, iinfo );
+        DEBUGMSGTL(("table:iterator:count", "next DP: %x %x %x (%d)\n",
+                                             ctx1, ctx2, vp2, i));
+        /* XXX - free context ? */
+    }
+           
+    /* XXX - final free context ? */
+    snmp_free_varbind( vp1 );
+    return i;
+}
+
+
+/* ==================================
+ *
+ * Iterator API: Index operations
+ *
+ * ================================== */
+
+
 /** @} */
