@@ -39,10 +39,6 @@
 #endif
 #include <errno.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
