@@ -46,10 +46,6 @@
 #include <arpa/inet.h>
 #endif
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #if HAVE_WINSOCK_H
 #include <winsock.h>
 #endif
