@@ -18,6 +18,8 @@
 #endif
 #if TIME_WITH_SYS_TIME
 # ifdef WIN32
+#  include <windows.h>
+#  include <errno.h>
 #  include <sys/timeb.h>
 # else
 #  include <sys/time.h>
@@ -100,12 +102,16 @@
 #if defined(CTL_HW) && defined(HW_PAGESIZE)
 #define USE_SYSCTL
 #endif
-#if defined(CTL_VM) && (defined(VM_METER) || defined(VM_UVMEXP))
+#if defined(CTL_VM) && (defined(VM_METER) || defined(VM_UVMEXP)) && !defined(darwin8)
 #define USE_SYSCTL_VM
 #endif
 #endif
 #endif                          /* ifndef dynix */
 
+#if defined(darwin8) /* This is to use host_statistics on OS X */
+#include <mach/mach.h>
+#endif
+
 #include "host_res.h"
 #include "hr_storage.h"
 #include "hr_filesys.h"
@@ -153,6 +159,9 @@
 #include <net-snmp/utilities.h>
 #include <net-snmp/output_api.h>
 
+#include <net-snmp/agent/net-snmp-agent-includes.h>
+#include <net-snmp/agent/hardware/memory.h>
+
 #if solaris2
 #include "kernel_sunos5.h"
 #endif
@@ -177,6 +186,14 @@ extern struct mnttab *HRFS_entry;
 #define HRFS_statfs	statvfs
 #define HRFS_HAS_FRSIZE STRUCT_STATVFS_HAS_F_FRSIZE
 
+#elif defined(WIN32)
+/* fake block size */
+#define FAKED_BLOCK_SIZE 512
+
+extern struct win_statfs *HRFS_entry;
+#define HRFS_statfs	win_statfs
+#define HRFS_mount	f_driveletter
+
 #elif defined(HAVE_STATVFS) && defined(__NetBSD__)
 
 extern struct statvfs *HRFS_entry;
@@ -185,7 +202,7 @@ extern int      fscount;
 #define HRFS_mount	f_mntonname
 #define HRFS_HAS_FRSIZE STRUCT_STATVFS_HAS_F_FRSIZE
 
-#elif defined(HAVE_STATVFS) && defined(STRUCT_STATVFS_HAS_MNT_DIR)
+#elif defined(HAVE_STATVFS)  && defined(STRUCT_STATVFS_HAS_MNT_DIR)
 
 extern struct mntent *HRFS_entry;
 extern int      fscount;
@@ -209,6 +226,10 @@ extern struct mntent *HRFS_entry;
 #define HRFS_HAS_FRSIZE STRUCT_STATFS_HAS_F_FRSIZE
 
 #endif
+	
+#if defined(darwin8) /* This is to use host_statistics() on OS X */
+mach_port_t myHost;
+#endif
 
 static int      physmem, pagesize;
 static void parse_storage_config(const char *, char *);
@@ -284,6 +305,7 @@ init_hr_storage(void)
         pagesize = pst_buf.page_size;
     }
 #else                           /* !USE_SYSCTL && !hpux10 && !hpux11 */
+#ifndef WIN32
 #ifdef HAVE_GETPAGESIZE
     pagesize = getpagesize();
 #elif defined(_SC_PAGESIZE)
@@ -304,6 +326,9 @@ init_hr_storage(void)
 #else
     pagesize = PAGESIZE;
 #endif
+#else /* WIN32 */
+	pagesize = 4096; /* Yes...Yes it does. */
+#endif
 #ifdef _SC_PHYS_PAGES
     physmem = sysconf(_SC_PHYS_PAGES);
 #else
@@ -321,6 +346,10 @@ init_hr_storage(void)
     auto_nlist(MBSTAT_SYMBOL, 0, 0);
 #endif
 
+#if defined(darwin8)
+    myHost = mach_host_self();
+#endif
+
     REGISTER_MIB("host/hr_storage", hrstore_variables, variable4,
                  hrstore_variables_oid);
 
@@ -490,6 +519,9 @@ var_hrstore(struct variable *vp,
     int             swap_total, swap_used;
 #elif defined(hpux10) || defined(hpux11)
     struct pst_dynamic pst_buf;
+#elif defined(darwin8)
+    vm_statistics_data_t vm_stat;
+    int count = HOST_VM_INFO_COUNT;
 #elif defined(TOTAL_MEMORY_SYMBOL) || defined(USE_SYSCTL_VM)
 #ifdef VM_UVMEXP
     struct uvmexp   uvmexp_totals;
@@ -547,6 +579,8 @@ really_try_next:
                     sysctl(mib, 2, &uvmexp_totals, &len, NULL, 0);
 #endif
                 }
+#elif defined(darwin8)
+		host_statistics(myHost,HOST_VM_INFO,&vm_stat,&count);
 #elif defined(hpux10) || defined(hpux11)
                 pstat_getdynamic(&pst_buf, sizeof(struct pst_dynamic), 1, 0);
 #elif defined(TOTAL_MEMORY_SYMBOL)
@@ -691,6 +725,18 @@ really_try_next:
             case HRS_TYPE_SWAP:
                 long_return = pst_buf.psd_vm;
                 break;
+#elif defined(darwin8)
+            case HRS_TYPE_MEM:
+                long_return = physmem;
+                break;
+            case HRS_TYPE_SWAP:
+                long_return = -1;
+	        break;
+#if defined(MBSTAT_SYMBOL)
+           case HRS_TYPE_MBUF:
+                long_return = mbstat.m_mbufs;
+                break;
+#endif
 #elif defined(TOTAL_MEMORY_SYMBOL) || defined(USE_SYSCTL_VM)
             case HRS_TYPE_MEM:
                 long_return = memory_totals.t_rm;
@@ -768,6 +814,18 @@ really_try_next:
             case HRS_TYPE_SWAP:
                 long_return = pst_buf.psd_avm;
                 break;
+#elif defined(darwin8)
+	    case HRS_TYPE_MEM:
+		long_return = vm_stat.active_count + vm_stat.inactive_count + vm_stat.wire_count;
+		break;
+	    case HRS_TYPE_SWAP:
+		long_return = -1;
+		break;
+#if defined(MBSTAT_SYMBOL)
+           case HRS_TYPE_MBUF:
+                long_return = mbstat.m_mbufs;
+                break;
+#endif
 #elif defined(TOTAL_MEMORY_SYMBOL) || defined(USE_SYSCTL_VM)
             case HRS_TYPE_MEM:
                 long_return = memory_totals.t_arm;
@@ -908,35 +966,34 @@ Get_Next_HR_Store(void)
 int
 linux_mem(int mem_type, int size_or_used)
 {
-    FILE           *fp;
-    char            buf[1024];
-    int             size = -1, free = -1, buffers = -1;
+    netsnmp_memory_info *mem;
 
-    if ((fp = fopen("/proc/meminfo", "r")) == NULL)
+    netsnmp_memory_load();
+    switch (mem_type) {
+    case HRS_TYPE_MEM:
+        mem = netsnmp_memory_get_byIdx( -1, 0 );
+        break;
+    case HRS_TYPE_SWAP:
+        mem = netsnmp_memory_get_byIdx( -2, 0 );
+        break;
+    case HRS_TYPE_MBUF:
+            /*
+             * The previous code reported total memory
+             * as "Memory Buffer" size
+             */
+        if (size_or_used == HRSTORE_SIZE)
+            mem = netsnmp_memory_get_byIdx( -1, 0 );
+        else
+            mem = netsnmp_memory_get_byIdx( -3, 0 );
+        return (mem ? mem->size : -1);
+    default:
         return -1;
-
-    while (fgets(buf, sizeof(buf), fp) != NULL) {
-       if ((!strncmp(buf, "MemTotal:", 9) && (mem_type == HRS_TYPE_MEM || mem_type == HRS_TYPE_MBUF)) ||
-            (!strncmp(buf, "SwapTotal:", 10) && mem_type == HRS_TYPE_SWAP))
-            sscanf(buf, "%*s %d", &size);
-
-       if ((!strncmp(buf, "MemFree:", 8) && mem_type == HRS_TYPE_MEM) ||
-            (!strncmp(buf, "SwapFree:", 9) && mem_type == HRS_TYPE_SWAP))
-            sscanf(buf, "%*s %d", &free);
-
-       if ((!strncmp(buf, "Buffers:", 8) && mem_type == HRS_TYPE_MBUF))
-		sscanf(buf, "%*s %d", &buffers);
     }
 
-    fclose(fp);
-
-    switch (mem_type) {
-        case HRS_TYPE_MBUF:
-            return (size_or_used == HRSTORE_SIZE ? size : buffers);
-        default:
-            return (size_or_used == HRSTORE_SIZE ? size : (size - free));
-    } /* switch */
-
+    if (mem)
+        return (size_or_used == HRSTORE_SIZE ?  mem->size
+                                             : (mem->size - mem->free));
+    return -1;
 }
 #endif
 
@@ -955,3 +1012,82 @@ sol_get_swapinfo(int *totalP, int *usedP)
     *usedP = ainfo.ani_resv;
 }
 #endif                          /* solaris2 */
+
+#ifdef WIN32
+char *win_realpath(const char *file_name, char *resolved_name)
+{
+	char szFile[_MAX_PATH + 1];
+	char *pszRet;
+ 	
+	pszRet = _fullpath(szFile, resolved_name, MAX_PATH);
+ 	
+	return pszRet;  
+}
+
+static int win_statfs (const char *path, struct win_statfs *buf)
+{
+    HINSTANCE h;
+    FARPROC f;
+    int retval = 0;
+    char tmp [MAX_PATH], resolved_path [MAX_PATH];
+    GetFullPathName(path, MAX_PATH, resolved_path, NULL);
+    /* TODO - Fix this! The realpath macro needs defined
+     * or rewritten into the function.
+     */
+    
+    win_realpath(path, resolved_path);
+    
+    if (!resolved_path)
+    	retval = - 1;
+    else {
+    	/* check whether GetDiskFreeSpaceExA is supported */
+        h = LoadLibraryA ("kernel32.dll");
+        if (h)
+			f = GetProcAddress (h, "GetDiskFreeSpaceExA");
+        else
+        	f = NULL;
+		
+        if (f) {
+			ULARGE_INTEGER bytes_free, bytes_total, bytes_free2;
+            if (!f (resolved_path, &bytes_free2, &bytes_total, &bytes_free)) {
+				errno = ENOENT;
+				retval = - 1;
+			} else {
+				buf -> f_bsize = FAKED_BLOCK_SIZE;
+				buf -> f_bfree = (bytes_free.QuadPart) / FAKED_BLOCK_SIZE;
+				buf -> f_files = buf -> f_blocks = (bytes_total.QuadPart) / FAKED_BLOCK_SIZE;
+				buf -> f_ffree = buf -> f_bavail = (bytes_free2.QuadPart) / FAKED_BLOCK_SIZE;
+			}
+		} else {
+			DWORD sectors_per_cluster, bytes_per_sector;
+			if (h) FreeLibrary (h);
+			if (!GetDiskFreeSpaceA (resolved_path, &sectors_per_cluster,
+					&bytes_per_sector, &buf -> f_bavail, &buf -> f_blocks)) {
+                errno = ENOENT;
+                retval = - 1;
+            } else {
+                buf -> f_bsize = sectors_per_cluster * bytes_per_sector;
+                buf -> f_files = buf -> f_blocks;
+                buf -> f_ffree = buf -> f_bavail;
+                buf -> f_bfree = buf -> f_bavail;
+            }
+		}
+		if (h) FreeLibrary (h);
+	}
+
+	/* get the FS volume information */
+	if (strspn (":", resolved_path) > 0) resolved_path [3] = '\0'; /* we want only the root */    
+	if (GetVolumeInformation (resolved_path, NULL, 0, &buf -> f_fsid, &buf -> f_namelen, 
+									NULL, tmp, MAX_PATH)) {
+		if (strcasecmp ("NTFS", tmp) == 0) {
+			buf -> f_type = NTFS_SUPER_MAGIC;
+		} else {
+			buf -> f_type = MSDOS_SUPER_MAGIC;
+		}
+	} else {
+		errno = ENOENT;
+		retval = - 1;
+	}
+	return retval;
+}
+#endif	/* WIN32 */
