@@ -1,7 +1,7 @@
 /*
  * arp data access header
  *
- * $Id: arp.h,v 1.1 2004/04/13 19:32:48 rstory Exp $
+ * $Id: arp.h,v 1.2 2005/10/29 03:31:29 rstory Exp $
  */
 /**---------------------------------------------------------------------*/
 /*
@@ -17,14 +17,14 @@
  *    HPUX11 had different reuirements than other HPUX, that should
  *    be handled in the *_hpux.h header file.
  */
-config_require(ip-mib/data_access/arp_common);
+config_require(ip-mib/data_access/arp_common)
 #if defined( linux )
-config_require(ip-mib/data_access/arp_linux);
+config_require(ip-mib/data_access/arp_linux)
 #else
 /*
  * couldn't determine the correct file!
  * require a bogus file to generate an error.
  */
-configure_require(ip-mib/data_access/arp-unknown-arch);
+config_require(ip-mib/data_access/arp-unknown-arch)
 #endif
 
