@@ -1,7 +1,7 @@
 /*
  * ipaddress data access header
  *
- * $Id: ipaddress_linux.h,v 1.1 2004/04/07 22:36:32 rstory Exp $
+ * $Id: ipaddress_linux.h,v 1.3 2005/11/02 17:07:28 rstory Exp $
  */
 /**---------------------------------------------------------------------*/
 /*
@@ -17,6 +17,6 @@
  *    HPUX11 had different reuirements than other HPUX, that should
  *    be handled in the *_hpux.h header file.
  */
-config_require(ip-mib/data_access/ipaddress_linux);
-config_require(ip-mib/data_access/ipaddress_ioctl);
+config_require(ip-mib/data_access/ipaddress_linux)
+config_require(ip-mib/data_access/ipaddress_ioctl)
 
