@@ -107,10 +107,6 @@
 int             strncasecmp(const char *s1, const char *s2, size_t n);
 #endif
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 #include <net-snmp/agent/auto_nlist.h>
@@ -224,7 +220,7 @@ extensible_parse_config(const char *token, char *cptr)
         /*
          * its a relocatable extensible mib 
          */
-        config_perror("Warning: relocatable 'exec' format will change in a future release - See 'NET-SNMP-EXTEND-MIB' for an alternative" );
+        config_perror("WARNING: This output format is being deprecated - Please use the 'extend' directive instead" );
         for (pp = &relocs, numrelocs++; *pp; pp = &((*pp)->next));
         (*pp) = ptmp;
         pp = &relocs; scount = numrelocs;
