@@ -37,10 +37,6 @@
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #include "struct.h"
 #include "util_funcs.h"
 #include "logmatch.h"
@@ -152,7 +148,7 @@ logmatch_parse_config(const char *token, char *cptr)
         /*
          * ------------------------------------ 
          * be careful this counter needs to be  
-         * resetfrom persistant storage         
+         * reset from persistent storage         
          * ------------------------------------ 
          */
 
