@@ -134,13 +134,11 @@ typedef long    fd_mask;
 #define FD_ZERO(p)      memset((p), 0, sizeof(*(p)))
 #endif
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
 
+#include <net-snmp/library/fd_event_manager.h>
+
 #include "m2m.h"
 #include <net-snmp/agent/mib_module_config.h>
 
@@ -188,11 +186,12 @@ int             Facility = LOG_DAEMON;
 #define AGENT_RUNNING 1
 #define AGENT_STOPPED 0
 int             agent_status = AGENT_STOPPED;
-LPTSTR          app_name = _T("Net-SNMP Agent");     /* Application Name */
-#else
-const char     *app_name = "snmpd";
+/* app_name_long used for Event Log (syslog), SCM, registry etc */
+LPTSTR          app_name_long = _T("Net-SNMP Agent");     /* Application Name */
 #endif
 
+const char     *app_name = "snmpd";
+
 extern int      netsnmp_running;
 extern char   **argvrestartp;
 extern char    *argvrestart;
@@ -397,36 +396,6 @@ SnmpTrapNodeDown(void)
      */
 }
 
-/*
- * dummy1 used to be stderr_log, but that was never set. So, ignore the
- * value here, instead of always disabling stderr logging (which might
- * have been set up using the new log handler methods).
- */
-static void
-setup_log(int restart, int dont_zero, int dummy1, int syslog_log, 
-	  char *logfile)
-{
-    static char logfile_s[PATH_MAX + 1] = { 0 };
-    static int dont_zero_s  = 0;
-    static int syslog_log_s = 0;
-
-    if (restart == 0) {
-	if (logfile != NULL) {
-	    strncpy(logfile_s, logfile, PATH_MAX);
-	}
-	dont_zero_s  = dont_zero;
-	syslog_log_s = syslog_log;
-    }
-
-    if (logfile_s[0]) {
-	snmp_enable_filelog(logfile_s, dont_zero_s);
-    }
-
-    if (syslog_log_s) {
-	snmp_enable_syslog_ident(app_name, Facility);
-    }
-}
-
 /*******************************************************************-o-******
  * main - Non Windows
  * SnmpDaemonMain - Windows to support windows service
@@ -450,14 +419,12 @@ SnmpDaemonMain(int argc, TCHAR * argv[])
 main(int argc, char *argv[])
 #endif
 {
-    char            options[128] = "aAc:CdD::fhHI:l:L:m:M:p:P:qrsS:UvV-:";
+    char            options[128] = "aAc:CdD::fhHI:l:L:m:M:n:p:P:qrsS:UvV-:";
     int             arg, i, ret;
-    int             dont_fork = 0;
-    int             dont_zero_log = 0;
-    int             syslog_log = 0;
+    int             dont_fork = 0, do_help = 0;
+    int             log_set = 0;
     int             uid = 0, gid = 0;
     int             agent_mode = -1;
-    char            logfile[PATH_MAX + 1] = { 0 };
     char           *cptr, **argvptr;
     char           *pid_file = NULL;
     char            option_compatability[] = "-Le";
@@ -474,7 +441,7 @@ main(int argc, char *argv[])
     for (i = getdtablesize() - 1; i > 2; --i) {
         (void) close(i);
     }
-#endif				/* #WIN32 */
+#endif /* #WIN32 */
     
     /*
      * register signals ASAP to prevent default action (usually core)
@@ -504,10 +471,6 @@ main(int argc, char *argv[])
     signal(SIGXFSZ, SnmpdCatchRandomSignal);
 #endif
 
-#ifdef LOGFILE
-    strncpy(logfile, LOGFILE, PATH_MAX);
-#endif
-
 #ifdef NO_ROOT_ACCESS
     /*
      * Default to no.  
@@ -549,7 +512,13 @@ main(int argc, char *argv[])
             argv[i] = option_compatability;            
     }
 
+#ifdef WIN32
+    snmp_log_syslogname(app_name_long);
+#else
     snmp_log_syslogname(app_name);
+#endif
+    netsnmp_ds_set_string(NETSNMP_DS_LIBRARY_ID,
+                          NETSNMP_DS_LIB_APPTYPE, app_name);
 
     /*
      * Now process options normally.  
@@ -572,7 +541,8 @@ main(int argc, char *argv[])
             break;
 
         case 'A':
-            dont_zero_log = 1;
+            netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID,
+                                   NETSNMP_DS_LIB_APPEND_LOGFILES, 1);
             break;
 
         case 'c':
@@ -620,14 +590,8 @@ main(int argc, char *argv[])
             break;
 
         case 'H':
-            netsnmp_ds_set_boolean(NETSNMP_DS_APPLICATION_ID, 
-				   NETSNMP_DS_AGENT_NO_ROOT_ACCESS, 1);
-            init_agent("snmpd");        /* register our .conf handlers */
-            init_mib_modules();
-            init_snmp("snmpd");
-            fprintf(stderr, "Configuration directives understood:\n");
-            read_config_print_usage("  ");
-            exit(0);
+            do_help = 1;
+            break;
 
         case 'I':
             if (optarg != NULL) {
@@ -646,7 +610,10 @@ main(int argc, char *argv[])
                             argv[0], PATH_MAX);
                     exit(1);
                 }
-                strncpy(logfile, optarg, PATH_MAX);
+                snmp_enable_filelog(optarg,
+                                    netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                                                           NETSNMP_DS_LIB_APPEND_LOGFILES));
+                log_set = 1;
             } else {
                 usage(argv[0]);
             }
@@ -656,6 +623,7 @@ main(int argc, char *argv[])
 	    if  (snmp_log_options( optarg, argc, argv ) < 0 ) {
                 usage(argv[0]);
             }
+            log_set = 1;
             break;
 
         case 'm':
@@ -674,6 +642,16 @@ main(int argc, char *argv[])
             }
             break;
 
+        case 'n':
+            if (optarg != NULL) {
+                app_name = optarg;
+                netsnmp_ds_set_string(NETSNMP_DS_LIBRARY_ID,
+                                      NETSNMP_DS_LIB_APPTYPE, app_name);
+            } else {
+                usage(argv[0]);
+            }
+            break;
+
         case 'P':
             printf("Warning: -P option is deprecated, use -p instead\n");
         case 'p':
@@ -695,7 +673,8 @@ main(int argc, char *argv[])
 
         case 's':
             printf("Warning: -s option is deprecated, use -Lsd instead\n");
-            syslog_log = 1;
+            snmp_enable_syslog();
+            log_set = 1;
             break;
 
         case 'S':
@@ -738,6 +717,8 @@ main(int argc, char *argv[])
                     fprintf(stderr, "invalid syslog facility: -S%c\n",*optarg);
                     usage(argv[0]);
                 }
+                snmp_enable_syslog_ident(snmp_log_syslogname(NULL), Facility);
+                log_set = 1;
             } else {
                 fprintf(stderr, "no syslog facility specified\n");
                 usage(argv[0]);
@@ -816,6 +797,17 @@ main(int argc, char *argv[])
         }
     }
 
+    if (do_help) {
+        netsnmp_ds_set_boolean(NETSNMP_DS_APPLICATION_ID, 
+                               NETSNMP_DS_AGENT_NO_ROOT_ACCESS, 1);
+        init_agent(app_name);        /* register our .conf handlers */
+        init_mib_modules();
+        init_snmp(app_name);
+        fprintf(stderr, "Configuration directives understood:\n");
+        read_config_print_usage("  ");
+        exit(0);
+    }
+
     if (optind < argc) {
         /*
          * There are optional transport addresses on the command line.  
@@ -844,7 +836,12 @@ main(int argc, char *argv[])
 					  NETSNMP_DS_AGENT_PORTS)));
     }
 
-    setup_log(0, dont_zero_log, 0, syslog_log, logfile);
+#ifdef LOGFILE
+    if (0 == log_set)
+        snmp_enable_filelog(LOGFILE,
+                            netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                                                   NETSNMP_DS_LIB_APPEND_LOGFILES));
+#endif
 
     /*
      * Initialize a argv set to the current for restarting the agent.   
@@ -900,13 +897,13 @@ main(int argc, char *argv[])
     }
 
     SOCK_STARTUP;
-    init_agent("snmpd");        /* do what we need to do first. */
+    init_agent(app_name);        /* do what we need to do first. */
     init_mib_modules();
 
     /*
      * start library 
      */
-    init_snmp("snmpd");
+    init_snmp(app_name);
 
     if ((ret = init_master_agent()) != 0) {
         /*
@@ -918,7 +915,7 @@ main(int argc, char *argv[])
     /*
      * Store persistent data immediately in case we crash later.  
      */
-    snmp_store("snmpd");
+    snmp_store(app_name);
 
     /*
      * Send coldstart trap if possible.  
@@ -1002,11 +999,10 @@ main(int argc, char *argv[])
     if (!netsnmp_ds_get_boolean(NETSNMP_DS_APPLICATION_ID, 
 				NETSNMP_DS_AGENT_QUIT_IMMEDIATELY))
         receive();
-#include "mib_module_shutdown.h"
     DEBUGMSGTL(("snmpd/main", "sending shutdown trap\n"));
     SnmpTrapNodeDown();
     DEBUGMSGTL(("snmpd/main", "Bye...\n"));
-    snmp_shutdown("snmpd");
+    snmp_shutdown(app_name);
 #ifdef SHUTDOWN_AGENT_CLEANLY /* broken code */
     /* these attempt to free all known memory, but result in double frees */
     shutdown_master_agent();
@@ -1067,8 +1063,7 @@ receive(void)
             snmp_log(LOG_INFO, "Reconfiguring daemon\n");
 	    /*  Stop and restart logging.  This allows logfiles to be
 		rotated etc.  */
-	    snmp_disable_log();
-	    setup_log(1, 0, 0, 0, NULL);
+	    netsnmp_logging_restart();
 	    snmp_log(LOG_INFO, "NET-SNMP version %s restarted\n",
 		     netsnmp_get_version());
             update_config();
@@ -1113,21 +1108,7 @@ receive(void)
         }
 #endif                          /* USING_SMUX_MODULE */
 
-        for (i = 0; i < external_readfdlen; i++) {
-            FD_SET(external_readfd[i], &readfds);
-            if (external_readfd[i] >= numfds)
-                numfds = external_readfd[i] + 1;
-        }
-        for (i = 0; i < external_writefdlen; i++) {
-            FD_SET(external_writefd[i], &writefds);
-            if (external_writefd[i] >= numfds)
-                numfds = external_writefd[i] + 1;
-        }
-        for (i = 0; i < external_exceptfdlen; i++) {
-            FD_SET(external_exceptfd[i], &exceptfds);
-            if (external_exceptfd[i] >= numfds)
-                numfds = external_exceptfd[i] + 1;
-        }
+        netsnmp_external_event_info(&numfds, &readfds, &writefds, &exceptfds);
 
     reselect:
         DEBUGMSGTL(("snmpd/select", "select( numfds=%d, ..., tvp=%p)\n",
@@ -1164,40 +1145,12 @@ receive(void)
                 }
             }
 #endif                          /* USING_SMUX_MODULE */
-
-            snmp_read(&readfds);
-
-            for (i = 0; count && (i < external_readfdlen); i++) {
-                if (FD_ISSET(external_readfd[i], &readfds)) {
-                    DEBUGMSGTL(("snmpd/select", "readfd[%d] = %d\n",
-                                i, external_readfd[i]));
-                    external_readfdfunc[i] (external_readfd[i],
-                                            external_readfd_data[i]);
-                    FD_CLR(external_readfd[i], &readfds);
-                    count--;
-                }
+            netsnmp_dispatch_external_events(&count, &readfds,
+                                           &writefds, &exceptfds);
+            /* If there are still events leftover, process them */
+            if (count > 0) {
+              snmp_read(&readfds);
             }
-            for (i = 0; count && (i < external_writefdlen); i++) {
-                if (FD_ISSET(external_writefd[i], &writefds)) {
-                    DEBUGMSGTL(("snmpd/select", "writefd[%d] = %d\n",
-                                i, external_writefd[i]));
-                    external_writefdfunc[i] (external_writefd[i],
-                                             external_writefd_data[i]);
-                    FD_CLR(external_writefd[i], &writefds);
-                    count--;
-                }
-            }
-            for (i = 0; count && (i < external_exceptfdlen); i++) {
-                if (FD_ISSET(external_exceptfd[i], &exceptfds)) {
-                    DEBUGMSGTL(("snmpd/select", "exceptfd[%d] = %d\n",
-                                i, external_exceptfd[i]));
-                    external_exceptfdfunc[i] (external_exceptfd[i],
-                                              external_exceptfd_data[i]);
-                    FD_CLR(external_exceptfd[i], &exceptfds);
-                    count--;
-                }
-            }
-
         } else
             switch (count) {
             case 0:
@@ -1304,7 +1257,7 @@ _tmain(int argc, TCHAR * argv[])
     /*
      * Define Service Name and Description, which appears in windows SCM 
      */
-    LPCTSTR         lpszServiceName = app_name;      /* Service Registry Name */
+    LPCTSTR         lpszServiceName = app_name_long;      /* Service Registry Name */
     LPCTSTR         lpszServiceDisplayName = _T("Net-SNMP Agent");       /* Display Name */
     LPCTSTR         lpszServiceDescription =
 #ifdef IFDESCR
