@@ -92,9 +92,11 @@ SOFTWARE.
 
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/agent/net-snmp-agent-includes.h>
+#include <net-snmp/library/fd_event_manager.h>
 #include "snmptrapd_handlers.h"
 #include "snmptrapd_log.h"
-#include "notification_log.h"
+#include "snmptrapd_auth.h"
+#include "notification-log-mib/notification_log.h"
 
 /*
  * Include winservice.h to support Windows Service
@@ -110,17 +112,8 @@ SOFTWARE.
 
 #if USE_LIBWRAP
 #include <tcpd.h>
-
-/* fix 706903 - these are defined in agent/snmp_agent.c *-
-int             allow_severity = LOG_INFO;
-int             deny_severity = LOG_WARNING;
- */
 #endif
 
-/*
- * #define NETSNMP_DS_APP_DONT_LOG 9 defined in notification_log.h 
- */
-
 #ifndef BSD4_3
 #define BSD4_2
 #endif
@@ -136,17 +129,11 @@ typedef long    fd_mask;
 #define FD_ZERO(p)      memset((p), 0, sizeof(*(p)))
 #endif
 
-extern int      netsnmp_running;
-
 char           *logfile = 0;
-int             Log = 0;
-int             Print = 0;
-int             Syslog = 0;
 int             SyslogTrap = 0;
 int             Event = 0;
 int             dropauth = 0;
 int             reconfig = 0;
-u_long          num_received = 0;
 char            ddefault_port[] = "udp:162";	/* Default default port */
 char           *default_port = ddefault_port;
 #if HAVE_GETPID
@@ -156,6 +143,8 @@ char           *default_port = ddefault_port;
 char           *trap1_fmt_str_remember = NULL;
 int             dofork = 1;
 
+extern int      netsnmp_running;
+
 /*
  * These definitions handle 4.2 systems without additional syslog facilities.
  */
@@ -206,11 +195,12 @@ int Facility = LOG_DAEMON;
 #define SNMPTRAPD_RUNNING 1
 #define SNMPTRAPD_STOPPED 0
 int             trapd_status = SNMPTRAPD_STOPPED;
-LPTSTR          app_name = _T("Net-SNMP Trap Handler");     /* Application Name */
-#else
-const char     *app_name = "snmptrapd";
+/* app_name_long used for Event Log (syslog), SCM, registry etc */
+LPTSTR          app_name_long = _T("Net-SNMP Trap Handler");     /* Application Name */
 #endif
 
+const char     *app_name = "snmptrapd";
+
 struct timeval  Now;
 
 void            trapd_update_config(void);
@@ -227,8 +217,8 @@ int __cdecl     _tmain(int argc, TCHAR * argv[]);
 int             main(int, char **);
 #endif
 
-#ifdef USING_AGENTX_SUBAGENT_MODULE
-void            init_subagent(void);
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+extern void            subagent_init(void);
 #endif
 
 void
@@ -346,6 +336,9 @@ usage(void)
     fprintf(stderr, "  \t\t\t  (followed -quiet to prevent message popups)\n");
 #endif
     fprintf(stderr, "  -v, --version\t\tdisplay version information\n");
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+    fprintf(stderr, "  -x ADDRESS\t\tuse ADDRESS as AgentX address\n");
+#endif
     fprintf(stderr,
             "  -O <OUTOPTS>\t\ttoggle options controlling output display\n");
     snmp_out_toggle_options_usage("\t\t\t", stderr);
@@ -354,7 +347,7 @@ usage(void)
     snmp_log_options_usage("\t\t\t", stderr);
     fprintf(stderr, "\n  Deprecated options:\n");
     fprintf(stderr, "  -o FILE\t\tuse -Lf <FILE> instead\n");
-    fprintf(stderr, "  -P\t\t\tuse -Le  instead\n");
+    fprintf(stderr, "  -P\t\t\tuse -f -Le  instead\n");
     fprintf(stderr, "  -s\t\t\tuse -Lsd instead\n");
     fprintf(stderr, "  -S d|i|0-7\t\tuse -Ls <facility> instead\n");
     fprintf(stderr, "  -u FILE\t\tuse -p <FILE> instead\n");
@@ -427,6 +420,7 @@ pre_parse(netsnmp_session * session, netsnmp_transport *transport,
 
         if (hosts_ctl("snmptrapd", STRING_UNKNOWN, 
 		      sbuf, STRING_UNKNOWN) == 0) {
+            DEBUGMSGTL(("snmptrapd:libwrap", "%s rejected", addr_string));
             SNMP_FREE(addr_string);
             return 0;
         }
@@ -435,6 +429,7 @@ pre_parse(netsnmp_session * session, netsnmp_transport *transport,
     } else {
         if (hosts_ctl("snmptrapd", STRING_UNKNOWN,
                       STRING_UNKNOWN, STRING_UNKNOWN) == 0) {
+            DEBUGMSGTL(("snmptrapd:libwrap", "[unknown] rejected"));
             return 0;
         }
     }
@@ -528,29 +523,27 @@ parse_config_logOption(const char *token, char *cptr)
   int my_argc = 0 ;
   char **my_argv = NULL;
 
-  if  (snmp_log_options( cptr, my_argc, my_argv ) >= 0 ) {
-    Log++;
-  }
+  snmp_log_options( cptr, my_argc, my_argv );
 }
 
 void
 parse_config_doNotFork(const char *token, char *cptr)
 {
-  if (atoi(cptr) == 1)
+  if (netsnmp_ds_parse_boolean(cptr) == 1)
     dofork = 0;
 }
 
 void
 parse_config_printEventNumbers(const char *token, char *cptr)
 {
-  if (atoi(cptr) == 1)
+  if (netsnmp_ds_parse_boolean(cptr) == 1)
     Event++;
 }
 
 void
 parse_config_ignoreAuthFailure(const char *token, char *cptr)
 {
-  if (atoi(cptr) == 1)
+  if (netsnmp_ds_parse_boolean(cptr) == 1)
     dropauth = 1;
 }
 
@@ -590,17 +583,18 @@ SnmpTrapdMain(int argc, TCHAR * argv[])
 main(int argc, char *argv[])
 #endif
 {
-    char            options[128] = "ac:CdD::efF:hHI:L:m:M:no:PqsS:tvO:-:";
+    char            options[128] = "ac:CdD::efF:hHI:L:m:M:no:PqsS:tvx:O:-:";
     netsnmp_session *sess_list = NULL, *ss = NULL;
     netsnmp_transport *transport = NULL;
     int             arg, i = 0, depmsg = 0;
     int             count, numfds, block;
-    fd_set          fdset;
+    fd_set          readfds,writefds,exceptfds;
     struct timeval  timeout, *tvp;
     char           *cp, *listen_ports = NULL;
 #if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
     int             agentx_subagent = 1;
 #endif
+    netsnmp_trapd_handler *traph;
 
 #ifdef SIGTERM
     signal(SIGTERM, term_handler);
@@ -620,7 +614,7 @@ main(int argc, char *argv[])
      */
     snmptrapd_register_configs( );
     init_usm_conf( "snmptrapd" );
-    register_config_handler("snmptrapd", "snmptrapdaddr",
+    register_config_handler("snmptrapd", "snmpTrapdAddr",
                             parse_trapd_address, free_trapd_address, "string");
 
     register_config_handler("snmptrapd", "doNotLogTraps",
@@ -644,7 +638,7 @@ main(int argc, char *argv[])
 
     register_config_handler("snmptrapd", "outputOption",
                             parse_config_outputOption, NULL, "string");
-    
+
 #ifdef WIN32
     setvbuf(stdout, NULL, _IONBF, BUFSIZ);
 #else
@@ -658,7 +652,11 @@ main(int argc, char *argv[])
     strcat(options, "p:u:");
 #endif
 
+#ifdef WIN32
+    snmp_log_syslogname(app_name_long);
+#else
     snmp_log_syslogname(app_name);
+#endif
 
     /*
      * Now process options normally.  
@@ -683,6 +681,11 @@ main(int argc, char *argv[])
             dropauth = 1;
             break;
 
+        case 'A':
+            netsnmp_ds_set_boolean(NETSNMP_DS_LIBRARY_ID,
+                                   NETSNMP_DS_LIB_APPEND_LOGFILES, 1);
+            break;
+
         case 'c':
             if (optarg != NULL) {
                 netsnmp_ds_set_string(NETSNMP_DS_LIBRARY_ID, 
@@ -730,8 +733,7 @@ main(int argc, char *argv[])
 
         case 'H':
             init_agent("snmptrapd");
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
-            init_subagent();
+#ifdef USING_NOTIFICATION_LOG_MIB_NOTIFICATION_LOG_MODULE
             init_notification_log();
 #endif
 #ifdef NETSNMP_EMBEDDED_PERL
@@ -826,10 +828,11 @@ main(int argc, char *argv[])
         case 'o':
             fprintf(stderr,
                     "Warning: -o option is deprecated; use -Lf <file> instead\n");
-            Print++;
             if (optarg != NULL) {
                 logfile = optarg;
-                snmp_enable_filelog(optarg, 0);
+                snmp_enable_filelog(optarg, 
+                                    netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                                                           NETSNMP_DS_LIB_APPEND_LOGFILES));
             } else {
                 usage();
                 exit(1);
@@ -851,7 +854,6 @@ main(int argc, char *argv[])
                 usage();
                 exit(1);
             }
-            Log++;
             break;
 
         case 'P':
@@ -859,14 +861,17 @@ main(int argc, char *argv[])
                     "Warning: -P option is deprecated; use -f -Le instead\n");
             dofork = 0;
             snmp_enable_stderrlog();
-            Print++;
             break;
 
         case 's':
             fprintf(stderr,
                     "Warning: -s option is deprecated; use -Lsd instead\n");
             depmsg = 1;
-            Syslog++;
+#ifdef WIN32
+            snmp_enable_syslog_ident(app_name_long, Facility);
+#else
+            snmp_enable_syslog_ident(app_name, Facility);
+#endif
             break;
 
         case 't':
@@ -893,6 +898,16 @@ main(int argc, char *argv[])
             exit(0);
             break;
 
+        case 'x':
+            if (optarg != NULL) {
+                netsnmp_ds_set_string(NETSNMP_DS_APPLICATION_ID,
+                                      NETSNMP_DS_AGENT_X_SOCKET, optarg);
+            } else {
+                usage();
+                exit(1);
+            }
+            break;
+
         default:
             fprintf(stderr, "invalid option: -%c\n", arg);
             usage();
@@ -931,20 +946,28 @@ main(int argc, char *argv[])
      * return value from these registration calls.
      * Don't try this at home, children!
      */
-    if (!Log && !Print) {
-        Syslog = 1;
-        netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER, syslog_handler);
+    if (0 == snmp_get_do_logging()) {
+        traph = netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER,
+                                               syslog_handler);
+        traph->authtypes = TRAP_AUTH_LOG;
     } else {
-        netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER, print_handler);
+        traph = netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER,
+                                               print_handler);
+        traph->authtypes = TRAP_AUTH_LOG;
     }
 
     if (Event) {
-        netsnmp_add_traphandler(event_handler, risingAlarm,
-                                    OID_LENGTH(risingAlarm));
-        netsnmp_add_traphandler(event_handler, fallingAlarm,
-                                    OID_LENGTH(fallingAlarm));
-        netsnmp_add_traphandler(event_handler, unavailableAlarm,
-                                    OID_LENGTH(unavailableAlarm));
+        traph = netsnmp_add_traphandler(event_handler, risingAlarm,
+                                        OID_LENGTH(risingAlarm));
+        traph->authtypes = TRAP_AUTH_LOG;
+
+        traph = netsnmp_add_traphandler(event_handler, fallingAlarm,
+                                        OID_LENGTH(fallingAlarm));
+        traph->authtypes = TRAP_AUTH_LOG;
+
+        traph = netsnmp_add_traphandler(event_handler, unavailableAlarm,
+                                        OID_LENGTH(unavailableAlarm));
+        traph->authtypes = TRAP_AUTH_LOG;
 	/* XXX - might be worth setting some "magic data"
 	 * in the traphandler structure that 'event_handler'
 	 * can use to avoid checking the trap OID values.
@@ -985,25 +1008,41 @@ main(int argc, char *argv[])
      * initialize local modules 
      */
     if (agentx_subagent) {
+#ifdef USING_SNMPV3_SNMPENGINE_MODULE
         extern void register_snmpEngine_scalars_context(const char *);
+#endif
+#ifdef USING_SNMPV3_USMUSER_MODULE
         extern void init_register_usmUser_context(const char *);
-        init_subagent();
+#endif
+        subagent_init();
+#ifdef USING_NOTIFICATION_LOG_MIB_NOTIFICATION_LOG_MODULE
+        /* register the notification log table */
         if (should_init("notificationLogMib")) {
-            netsnmp_add_global_traphandler(NETSNMPTRAPD_POST_HANDLER,
-                                           notification_handler);
-            /* register the notification log table */
+            netsnmp_ds_set_string(NETSNMP_DS_APPLICATION_ID,
+                              NETSNMP_DS_NOTIF_LOG_CTX,
+                              "snmptrapd");
+            traph = netsnmp_add_global_traphandler(NETSNMPTRAPD_POST_HANDLER,
+                                                   notification_handler);
+            traph->authtypes = TRAP_AUTH_LOG;
             init_notification_log();
         }
-
+#endif
+#ifdef USING_SNMPV3_SNMPENGINE_MODULE
         /*
          * register scalars from SNMP-FRAMEWORK-MIB::snmpEngineID group;
          * allows engineID probes via the master agent under the
          * snmptrapd context
          */
         register_snmpEngine_scalars_context("snmptrapd");
+#endif
 
+        /* register our authorization handler */
+        init_netsnmp_trapd_auth();
+
+#ifdef USING_SNMPV3_USMUSER_MODULE
         /* register ourselves as having a USM user database */
         init_register_usmUser_context("snmptrapd");
+#endif
     }
 #endif
 
@@ -1040,6 +1079,18 @@ main(int argc, char *argv[])
          */
         netsnmp_running = 0;
     }
+
+    /*
+     * if no logging options on command line or in conf files, use syslog
+     */
+    if (0 == snmp_get_do_logging()) {
+#ifdef WIN32
+        snmp_enable_syslog_ident(app_name_long, Facility);
+#else
+        snmp_enable_syslog_ident(app_name, Facility);
+#endif        
+    }
+
 #ifndef WIN32
     /*
      * fork the process to the background if we are not printing to stderr 
@@ -1088,25 +1139,11 @@ main(int argc, char *argv[])
     }
 #endif
 
-    if (Syslog) {
-        snmp_enable_syslog_ident(app_name, Facility);
-        snmp_log(LOG_INFO, "Starting snmptrapd %s\n", netsnmp_get_version());
-	if (depmsg) {
-	    snmp_log(LOG_WARNING, "-s and -S options are deprecated; use -Ls <facility> instead\n");
-	}
+    snmp_log(LOG_INFO, "NET-SNMP version %s\n", netsnmp_get_version());
+    if (depmsg) {
+        snmp_log(LOG_WARNING, "-s and -S options are deprecated; use -Ls <facility> instead\n");
     }
-    if (Print || Log) {
-        struct tm      *tm;
-        time_t          timer;
-        time(&timer);
-        tm = localtime(&timer);
-        snmp_log(LOG_INFO,
-                "%.4d-%.2d-%.2d %.2d:%.2d:%.2d NET-SNMP version %s Started.\n",
-                 tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-                 tm->tm_hour, tm->tm_min, tm->tm_sec,
-                 netsnmp_get_version());
-    }
-
+    
     SOCK_STARTUP;
 
     if (listen_ports)
@@ -1148,9 +1185,7 @@ main(int argc, char *argv[])
                  */
                 snmptrapd_close_sessions(sess_list);
                 netsnmp_transport_free(transport);
-                if (Syslog) {
-                    snmp_log(LOG_ERR, "couldn't open snmp - %m");
-                }
+                snmp_log(LOG_ERR, "couldn't open snmp - %m");
                 SOCK_CLEANUP;
                 exit(1);
             } else {
@@ -1181,29 +1216,15 @@ main(int argc, char *argv[])
 #endif
     while (netsnmp_running) {
         if (reconfig) {
-            if (Print || Log) {
-                struct tm      *tm;
-                time_t          timer;
-                time(&timer);
-                tm = localtime(&timer);
-
                 /*
                  * If we are logging to a file, receipt of SIGHUP also
                  * indicates the the log file should be closed and
                  * re-opened.  This is useful for users that want to
                  * rotate logs in a more predictable manner.
                  */
-                if (logfile)
-                    snmp_enable_filelog(logfile, 1);
-
-                snmp_log(LOG_INFO,
-                         "%.4d-%.2d-%.2d %.2d:%.2d:%.2d NET-SNMP version %s Reconfigured.\n",
-                         tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-                         tm->tm_hour, tm->tm_min, tm->tm_sec,
+                netsnmp_logging_restart();
+                snmp_log(LOG_INFO, "NET-SNMP version %s restarted\n",
                          netsnmp_get_version());
-            }
-            if (Syslog)
-                snmp_log(LOG_INFO, "Snmptrapd reconfiguring");
             trapd_update_config();
             if (trap1_fmt_str_remember) {
                 free_trap1_fmt();
@@ -1212,18 +1233,27 @@ main(int argc, char *argv[])
             reconfig = 0;
         }
         numfds = 0;
-        FD_ZERO(&fdset);
+        FD_ZERO(&readfds);
+        FD_ZERO(&writefds);
+        FD_ZERO(&exceptfds);
         block = 0;
         tvp = &timeout;
         timerclear(tvp);
         tvp->tv_sec = 5;
-        snmp_select_info(&numfds, &fdset, tvp, &block);
+        snmp_select_info(&numfds, &readfds, tvp, &block);
         if (block == 1)
             tvp = NULL;         /* block without timeout */
-        count = select(numfds, &fdset, 0, 0, tvp);
+        netsnmp_external_event_info(&numfds, &readfds, &writefds, &exceptfds);
+        count = select(numfds, &readfds, &writefds, &exceptfds, tvp);
         gettimeofday(&Now, 0);
         if (count > 0) {
-            snmp_read(&fdset);
+            netsnmp_dispatch_external_events(&count, &readfds, &writefds,
+                                             &exceptfds);
+            /* If there are any more events after external events, then
+             * try SNMP events. */
+            if (count > 0) {
+                snmp_read(&readfds);
+            }
         } else
             switch (count) {
             case 0:
@@ -1242,7 +1272,7 @@ main(int argc, char *argv[])
 	run_alarms();
     }
 
-    if (Print || Log) {
+    if (snmp_get_do_logging()) {
         struct tm      *tm;
         time_t          timer;
         time(&timer);
@@ -1252,10 +1282,8 @@ main(int argc, char *argv[])
                  tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday, tm->tm_hour,
                  tm->tm_min, tm->tm_sec, netsnmp_get_version());
     }
-    if (Syslog) {
-        snmp_log(LOG_INFO, "Stopping snmptrapd");
-    }
-
+    snmp_log(LOG_INFO, "Stopping snmptrapd");
+    
     snmptrapd_close_sessions(sess_list);
     snmp_shutdown("snmptrapd");
 #ifdef WIN32SERVICE
@@ -1276,6 +1304,7 @@ void
 trapd_update_config(void)
 {
     free_config();
+    vacm_standard_views(0,0,NULL,NULL);
     read_configs();
 }
 
@@ -1309,7 +1338,7 @@ _tmain(int argc, TCHAR * argv[])
     /*
      * Define Service Name and Description, which appears in windows SCM 
      */
-    LPCTSTR         lpszServiceName = app_name;      /* Service Registry Name */
+    LPCTSTR         lpszServiceName = app_name_long;      /* Service Registry Name */
     LPCTSTR         lpszServiceDisplayName = _T("Net-SNMP Trap Handler");       /* Display Name */
     LPCTSTR         lpszServiceDescription =
 #ifdef IFDESCR
