@@ -14,6 +14,7 @@ struct netsnmp_trapd_handler_s {
      char *token;		/* Or an array of tokens? */
      char *format;		/* Formatting string */
      int   version;		/* ??? */
+     int   authtypes;
      Netsnmp_Trap_Handler *handler;
      void *handler_data;
 
@@ -28,6 +29,7 @@ Netsnmp_Trap_Handler   print_handler;
 Netsnmp_Trap_Handler   command_handler;
 Netsnmp_Trap_Handler   event_handler;
 Netsnmp_Trap_Handler   forward_handler;
+Netsnmp_Trap_Handler   notification_handler;
 
 void free_trap1_fmt(void);
 void free_trap2_fmt(void);
