@@ -16,6 +16,13 @@
 #define NETSNMP_DS_AGENT_NO_CONNECTION_WARNINGS 6 /* 1 = !see !connect msgs */
 #define NETSNMP_DS_AGENT_LEAVE_PIDFILE  7       /* 1 = leave PID file on exit */
 #define NETSNMP_DS_AGENT_NO_CACHING     8       /* 1 = disable netsnmp_cache */
+#define NETSNMP_DS_AGENT_STRICT_DISMAN  9       /* 1 = "correct" object ordering */
+#define NETSNMP_DS_AGENT_DONT_RETAIN_NOTIFICATIONS 10   /* 1 = disable trap logging */
+#define NETSNMP_DS_APP_DONT_LOG         NETSNMP_DS_AGENT_DONT_RETAIN_NOTIFICATIONS /* compat */
+
+/* WARNING: The trap receiver uses DS flags and must not conflict with
+   these!  If you use a value above 15, change the minimimum DS bool
+   value in snmptrapd_log.h */
 
 /*
  * strings 
@@ -26,6 +33,7 @@
 #define NETSNMP_DS_AGENT_INTERNAL_SECNAME  3    /* used by disman/mteTriggerTable. */
 #define NETSNMP_DS_AGENT_PERL_INIT_FILE    4    /* used by embedded perl */
 #define NETSNMP_DS_SMUX_SOCKET    5     /* ip:port socket addr */
+#define NETSNMP_DS_NOTIF_LOG_CTX  6     /* "" | "snmptrapd" */
 
 /*
  * integers 
@@ -41,5 +49,7 @@
 #define NETSNMP_DS_AGENT_X_SOCK_USER     8      /* ownership for the   */
 #define NETSNMP_DS_AGENT_X_SOCK_GROUP    9      /*     AgentX socket   */
 #define NETSNMP_DS_AGENT_CACHE_TIMEOUT  10      /* default cache timeout */
+#define NETSNMP_DS_AGENT_INTERNAL_VERSION  11   /* used by internal queries */
+#define NETSNMP_DS_AGENT_INTERNAL_SECLEVEL 12   /* used by internal queries */
 
 #endif
