@@ -7,7 +7,7 @@
 #     modify it under the same terms as Perl itself.
 
 package SNMP;
-$VERSION = '5.0203';   # current release version number
+$VERSION = '5.0301';   # current release version number
 
 require Exporter;
 require DynaLoader;
@@ -537,11 +537,13 @@ sub new {
 
    SNMP::initMib($SNMP::auto_init_mib); # ensures that *some* mib is loaded
 
-   $this->{UseLongNames} ||= $SNMP::use_long_names;
-   $this->{UseSprintValue} ||= $SNMP::use_sprint_value;
-   $this->{BestGuess} ||= $SNMP::best_guess;
-   $this->{UseEnums} ||= $SNMP::use_enums;
-   $this->{UseNumeric} ||= $SNMP::use_numeric;
+   $this->{UseLongNames} = $SNMP::use_long_names 
+       unless exists $this->{UseLongNames};
+   $this->{UseSprintValue} = $SNMP::use_sprint_value 
+       unless exists $this->{UseSprintValue};
+   $this->{BestGuess} = $SNMP::best_guess unless exists $this->{BestGuess};
+   $this->{UseEnums} = $SNMP::use_enums unless exists $this->{UseEnums};
+   $this->{UseNumeric} = $SNMP::use_numeric unless exists $this->{UseNumeric};
 
    # Force UseLongNames if UseNumeric is in use.
    $this->{UseLongNames}++  if $this->{UseNumeric};
@@ -563,11 +565,13 @@ sub update {
 
    @$this{keys %new_fields} = values %new_fields;
 
-   $this->{UseLongNames} ||= $SNMP::use_long_names;
-   $this->{UseSprintValue} ||= $SNMP::use_sprint_value;
-   $this->{BestGuess} ||= $SNMP::best_guess;
-   $this->{UseEnums} ||= $SNMP::use_enums;
-   $this->{UseNumeric} ||= $SNMP::use_numeric;
+   $this->{UseLongNames} = $SNMP::use_long_names 
+       unless exists $this->{UseLongNames};
+   $this->{UseSprintValue} = $SNMP::use_sprint_value 
+       unless exists $this->{UseSprintValue};
+   $this->{BestGuess} = $SNMP::best_guess unless exists $this->{BestGuess};
+   $this->{UseEnums} = $SNMP::use_enums unless exists $this->{UseEnums};
+   $this->{UseNumeric} = $SNMP::use_numeric unless exists $this->{UseNumeric};
 
    # Force UseLongNames if UseNumeric is in use.
    $this->{UseLongNames}++  if $this->{UseNumeric};
@@ -709,6 +713,12 @@ sub gettable {
 	push @$varbinds, [$c];
 	push @$stopconds, $c;
     }
+
+    if ($#$varbinds == -1) {
+	print STDERR "ack: gettable failed to find any columns to look for.\n";
+	return;
+    }
+
     $vbl = $varbinds;
 	
     my $repeatcount;
@@ -716,6 +726,8 @@ sub gettable {
 	$repeatcount = 1;
     } elsif ($options->{'repeat'}) {
 	$repeatcount = $options->{'repeat'};
+    } elsif ($#$varbinds == -1) {
+	$repeatcount = 1;
     } else {
 	# experimentally determined maybe guess at a best repeat value
 	# 1000 bytes max (safe), 30 bytes average for encoding of the
