@@ -12,6 +12,7 @@ static netsnmp_container *containers = NULL;
 typedef struct container_type_s {
    const char                 *name;
    netsnmp_factory            *factory;
+   netsnmp_container_compare  *compare;
 } container_type;
 
 netsnmp_factory *
@@ -65,6 +66,14 @@ netsnmp_container_init_list(void)
                                netsnmp_container_get_factory("sorted_singly_linked_list"));
     netsnmp_container_register("ssll_container",
                                netsnmp_container_get_factory("sorted_singly_linked_list"));
+
+    netsnmp_container_register_with_compare
+        ("string", netsnmp_container_get_factory("binary_array"),
+         netsnmp_compare_cstring);
+    netsnmp_container_register_with_compare
+        ("string:binary_array", netsnmp_container_get_factory("binary_array"),
+         netsnmp_compare_cstring);
+
 }
 
 void
@@ -77,7 +86,7 @@ netsnmp_container_free_list(void)
     /*
      * free memory used by each factory entry
      */
-    CONTAINER_FOR_EACH(containers, _factory_free, NULL);
+    CONTAINER_FOR_EACH(containers, ((netsnmp_container_obj_func *)_factory_free), NULL);
 
     /*
      * free factory container
@@ -87,11 +96,12 @@ netsnmp_container_free_list(void)
 }
 
 int
-netsnmp_container_register(const char* name, netsnmp_factory *f)
+netsnmp_container_register_with_compare(const char* name, netsnmp_factory *f,
+                                        netsnmp_container_compare  *c)
 {
     container_type *ct, tmp;
 
-    tmp.name = name;
+    tmp.name = (char *)name;
     ct = CONTAINER_FIND(containers, &tmp);
     if (NULL!=ct) {
         DEBUGMSGT(("container_registry",
@@ -104,6 +114,7 @@ netsnmp_container_register(const char* name, netsnmp_factory *f)
             return -1;
         ct->name = strdup(name);
         ct->factory = f;
+        ct->compare = c;
         CONTAINER_INSERT(containers, ct);
     }
     DEBUGMSGT(("container_registry", "registered container factory %s (%s)\n",
@@ -112,6 +123,12 @@ netsnmp_container_register(const char* name, netsnmp_factory *f)
     return 0;
 }
 
+int
+netsnmp_container_register(const char* name, netsnmp_factory *f)
+{
+    return netsnmp_container_register_with_compare(name, f, NULL);
+}
+
 /*------------------------------------------------------------------
  */
 netsnmp_factory *
@@ -148,14 +165,55 @@ netsnmp_container_find_factory(const char *type_list)
     return f;
 }
 
+/*------------------------------------------------------------------
+ */
+static container_type *
+netsnmp_container_get_ct(const char *type)
+{
+    container_type ct;
+    
+    ct.name = type;
+    return CONTAINER_FIND(containers, &ct);
+}
+
+static container_type *
+netsnmp_container_find_ct(const char *type_list)
+{
+    container_type    *ct = NULL;
+    char              *list, *entry;
+    char              *st;
+
+    if (NULL==type_list)
+        return NULL;
+
+    list = strdup(type_list);
+    entry = strtok_r(list, ":", &st);
+    while(entry) {
+        ct = netsnmp_container_get_ct(entry);
+        if (NULL != ct)
+            break;
+        entry = strtok_r(NULL, ":", &st);
+    }
+
+    free(list);
+    return ct;
+}
+
+
+
 /*------------------------------------------------------------------
  */
 netsnmp_container *
 netsnmp_container_get(const char *type)
 {
-    netsnmp_factory *f = netsnmp_container_get_factory(type);
-    if (f)
-        return f->produce();
+    netsnmp_container *c;
+    container_type *ct = netsnmp_container_get_ct(type);
+    if (ct) {
+        c = ct->factory->produce();
+        if (c && ct->compare)
+            c->compare = ct->compare;
+        return c;
+    }
 
     return NULL;
 }
@@ -165,14 +223,18 @@ netsnmp_container_get(const char *type)
 netsnmp_container *
 netsnmp_container_find(const char *type)
 {
-    netsnmp_factory *f = netsnmp_container_find_factory(type);
-    netsnmp_container *c = f ? f->produce() : NULL;
+    container_type *ct = netsnmp_container_find_ct(type);
+    netsnmp_container *c = ct ? ct->factory->produce() : NULL;
 
     /*
      * provide default compare
      */
-    if (c && (NULL == c->compare))
-        c->compare = netsnmp_compare_netsnmp_index;
+    if (c) {
+        if (ct->compare)
+            c->compare = ct->compare;
+        else if (NULL == c->compare)
+            c->compare = netsnmp_compare_netsnmp_index;
+    }
 
     return c;
 }
@@ -201,7 +263,7 @@ netsnmp_container_add_index(netsnmp_container *primary,
 
 /*------------------------------------------------------------------
  * These functions should EXACTLY match the inline version in
- * container.h. If you chance one, change them both.
+ * container.h. If you change one, change them both.
  */
 int CONTAINER_INSERT(netsnmp_container *x, const void *k)
 { 
@@ -210,20 +272,23 @@ int CONTAINER_INSERT(netsnmp_container *x, const void *k)
     /** start at first container */
     while(x->prev)
         x = x->prev;
-    while(x) {
+    for(; x; x = x->next) {
+        if ((NULL != x->insert_filter) &&
+            (x->insert_filter(x,k) == 1))
+            continue;
         rc2 = x->insert(x,k);
         if (rc2) {
-            snmp_log(LOG_ERR,"error on subcontainer insert (%d)\n", rc2);
+            snmp_log(LOG_ERR,"error on subcontainer '%s' insert (%d)\n",
+                     x->container_name ? x->container_name : "", rc2);
             rc = rc2;
         }
-        x = x->next;
     }
     return rc;
 }
 
 /*------------------------------------------------------------------
  * These functions should EXACTLY match the inline version in
- * container.h. If you chance one, change them both.
+ * container.h. If you change one, change them both.
  */
 int CONTAINER_REMOVE(netsnmp_container *x, const void *k)
 {
@@ -234,7 +299,8 @@ int CONTAINER_REMOVE(netsnmp_container *x, const void *k)
         x = x->next;
     while(x) {
         rc2 = x->remove(x,k);
-        if (rc2) {
+        /** ignore remove errors if there is a filter in place */
+        if ((rc2) && (NULL == x->insert_filter)) {
             snmp_log(LOG_ERR,"error on subcontainer remove (%d)\n", rc2);
             rc = rc2;
         }
@@ -246,7 +312,7 @@ int CONTAINER_REMOVE(netsnmp_container *x, const void *k)
 
 /*------------------------------------------------------------------
  * These functions should EXACTLY match the inline version in
- * container.h. If you chance one, change them both.
+ * container.h. If you change one, change them both.
  */
 int CONTAINER_FREE(netsnmp_container *x)
 {
