@@ -1,15 +1,11 @@
 /*
  * netsnmp_data_list.c
  *
- * $Id: data_list.c,v 5.7.2.1 2006/01/25 16:28:11 dts12 Exp $
+ * $Id: data_list.c,v 5.11.2.1 2006/01/25 16:27:41 dts12 Exp $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
 
-#if HAVE_DMALLOC_H
-#include <dmalloc.h>
-#endif
-
 /*
  * prototypes
  */
@@ -64,7 +60,11 @@ NETSNMP_INLINE netsnmp_data_list *
 netsnmp_create_data_list(const char *name, void *data,
                          Netsnmp_Free_List_Data * beer)
 {
-    netsnmp_data_list *node = SNMP_MALLOC_TYPEDEF(netsnmp_data_list);
+    netsnmp_data_list *node;
+    
+    if (!name)
+        return NULL;
+    node = SNMP_MALLOC_TYPEDEF(netsnmp_data_list);
     if (!node)
         return NULL;
     node->name = strdup(name);
@@ -137,7 +137,12 @@ NETSNMP_INLINE netsnmp_data_list *
 netsnmp_data_list_add_data(netsnmp_data_list **head, const char *name,
                            void *data, Netsnmp_Free_List_Data * beer)
 {
-    netsnmp_data_list *node = netsnmp_create_data_list(name, data, beer);
+    netsnmp_data_list *node;
+    if (!name) {
+        snmp_log(LOG_ERR,"no name provided.");
+        return NULL;
+    }
+    node = netsnmp_create_data_list(name, data, beer);
     if(NULL == node) {
         snmp_log(LOG_ERR,"could not allocate memory for node.");
         return NULL;
@@ -156,6 +161,8 @@ netsnmp_data_list_add_data(netsnmp_data_list **head, const char *name,
 NETSNMP_INLINE void    *
 netsnmp_get_list_data(netsnmp_data_list *head, const char *name)
 {
+    if (!name)
+        return NULL;
     for (; head; head = head->next)
         if (head->name && strcmp(head->name, name) == 0)
             break;
@@ -172,6 +179,8 @@ netsnmp_get_list_data(netsnmp_data_list *head, const char *name)
 NETSNMP_INLINE netsnmp_data_list    *
 netsnmp_get_list_node(netsnmp_data_list *head, const char *name)
 {
+    if (!name)
+        return NULL;
     for (; head; head = head->next)
         if (head->name && strcmp(head->name, name) == 0)
             break;
@@ -189,6 +198,8 @@ int
 netsnmp_remove_list_node(netsnmp_data_list **realhead, const char *name)
 {
     netsnmp_data_list *head, *prev;
+    if (!name)
+        return 1;
     for (head = *realhead, prev = NULL; head;
          prev = head, head = head->next) {
         if (head->name && strcmp(head->name, name) == 0) {
@@ -207,7 +218,7 @@ netsnmp_remove_list_node(netsnmp_data_list **realhead, const char *name)
 /** used to store registered save/parse handlers (specifically, parsing info) */
 static netsnmp_data_list *saveHead;
 
-/** registers to store a data_list set of data at persistant storage time
+/** registers to store a data_list set of data at persistent storage time
  *
  * @param datalist the data to be saved
  * @param type the name of the application to save the data as.  If left NULL the default application name that was registered during the init_snmp call will be used (recommended).
