@@ -840,11 +840,13 @@ read_configs(void)
 
     DEBUGMSGTL(("read_config", "reading normal configuration tokens\n"));
 
-    if (!netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID, 
-				NETSNMP_DS_LIB_DONT_READ_CONFIGS)) {
-        read_config_files(NORMAL_CONFIG);
+    if ((NULL != optional_config) && (*optional_config == '-')) {
+        read_configs_optional(++optional_config, NORMAL_CONFIG);
+        optional_config = NULL; /* clear, so we don't read them twice */
     }
 
+    read_config_files(NORMAL_CONFIG);
+
     /*
      * do this even when the normal above wasn't done 
      */
@@ -867,11 +869,13 @@ read_premib_configs(void)
 
     DEBUGMSGTL(("read_config", "reading premib configuration tokens\n"));
 
-    if (!netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID, 
-				NETSNMP_DS_LIB_DONT_READ_CONFIGS)) {
-        read_config_files(PREMIB_CONFIG);
+    if ((NULL != optional_config) && (*optional_config == '-')) {
+        read_configs_optional(++optional_config, PREMIB_CONFIG);
+        optional_config = NULL; /* clear, so we don't read them twice */
     }
 
+    read_config_files(PREMIB_CONFIG);
+
     if (NULL != optional_config)
         read_configs_optional(optional_config, PREMIB_CONFIG);
 
@@ -963,7 +967,10 @@ get_persistent_directory()
 {
     if (NULL == netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				      NETSNMP_DS_LIB_PERSISTENT_DIR)) {
-        set_persistent_directory(PERSISTENT_DIRECTORY);
+        char *persdir = netsnmp_getenv("SNMP_PERSISTENT_DIR");
+        if (NULL == persdir)
+            persdir = PERSISTENT_DIRECTORY;
+        set_persistent_directory(persdir);
     }
     return (netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				  NETSNMP_DS_LIB_PERSISTENT_DIR));
@@ -1006,6 +1013,85 @@ get_temp_file_pattern()
 				  NETSNMP_DS_LIB_TEMP_FILE_PATTERN));
 }
 
+/**
+ * utility routine for read_config_files
+ */
+static void
+read_config_files_in_path(const char *path, struct config_files *ctmp,
+                          int when, const char *perspath, const char *persfile)
+{
+    int             done, j;
+    char            configfile[300];
+    char           *cptr1, *cptr2, *envconfpath;
+    struct stat     statbuf;
+
+    if ((NULL == path) || (NULL == ctmp))
+        return;
+
+    envconfpath = strdup(path);
+
+    DEBUGMSGTL(("read_config", " config path used for %s:%s (persistent path:%s)\n",
+                ctmp->fileHeader, envconfpath, perspath));
+    cptr1 = cptr2 = envconfpath;
+    done = 0;
+    while (*cptr2 != 0) {
+        while (*cptr1 != 0 && *cptr1 != ENV_SEPARATOR_CHAR)
+            cptr1++;
+        if (*cptr1 == 0)
+            done = 1;
+        else
+            *cptr1 = 0;
+        /*
+         * for proper persistent storage retrival, we need to read old backup
+         * copies of the previous storage files.  If the application in
+         * question has died without the proper call to snmp_clean_persistent,
+         * then we read all the configuration files we can, starting with
+         * the oldest first.
+         */
+        if (strncmp(cptr2, perspath, strlen(perspath)) == 0 ||
+            (persfile != NULL &&
+             strncmp(cptr2, persfile, strlen(persfile)) == 0)) {
+            /*
+             * limit this to the known storage directory only 
+             */
+            for (j = 0; j <= MAX_PERSISTENT_BACKUPS; j++) {
+                snprintf(configfile, sizeof(configfile),
+                         "%s/%s.%d.conf", cptr2,
+                         ctmp->fileHeader, j);
+                configfile[ sizeof(configfile)-1 ] = 0;
+                if (stat(configfile, &statbuf) != 0) {
+                    /*
+                     * file not there, continue 
+                     */
+                    break;
+                } else {
+                    /*
+                     * backup exists, read it 
+                     */
+                    DEBUGMSGTL(("read_config_files",
+                                "old config file found: %s, parsing\n",
+                                configfile));
+                    read_config(configfile, ctmp->start, when);
+                }
+            }
+        }
+        snprintf(configfile, sizeof(configfile),
+                 "%s/%s.conf", cptr2, ctmp->fileHeader);
+        configfile[ sizeof(configfile)-1 ] = 0;
+        read_config(configfile, ctmp->start, when);
+        snprintf(configfile, sizeof(configfile),
+                 "%s/%s.local.conf", cptr2, ctmp->fileHeader);
+        configfile[ sizeof(configfile)-1 ] = 0;
+        read_config(configfile, ctmp->start, when);
+
+        if(done)
+            break;
+
+        cptr2 = ++cptr1;
+    }
+    SNMP_FREE(envconfpath);
+}
+
 /*******************************************************************-o-******
  * read_config_files
  *
@@ -1038,118 +1124,59 @@ get_temp_file_pattern()
 void
 read_config_files(int when)
 {
-    int             i, j;
-    char            configfile[300];
-    char           *envconfpath, *persfile;
-    const char     *confpath, *perspath;
-    char           *cptr1, *cptr2;
-    char            defaultPath[SPRINT_MAX_LEN];
-
+    const char     *confpath, *perspath, *persfile, *envconfpath;
     struct config_files *ctmp = config_files;
-    struct config_line *ltmp;
-    struct stat     statbuf;
 
     if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
-                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)) return;
+                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)
+     || netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                               NETSNMP_DS_LIB_DISABLE_CONFIG_LOAD)) return;
 
     config_errors = 0;
 
     if (when == PREMIB_CONFIG)
         free_config();
 
+    /*
+     * these shouldn't change
+     */
     confpath = get_configuration_directory();
+    persfile = netsnmp_getenv("SNMP_PERSISTENT_FILE");
+    envconfpath = netsnmp_getenv("SNMPCONFPATH");
 
     /*
      * read all config file types 
      */
     for (; ctmp != NULL; ctmp = ctmp->next) {
 
-        ltmp = ctmp->start;
-
-        /*
-         * persistent path can change via conf file, so make sure
-         * we have a valid pointer.
-         */
-        perspath = get_persistent_directory();
-
         /*
          * read the config files 
          */
-        if ((envconfpath = netsnmp_getenv("SNMPCONFPATH")) == NULL) {
-            snprintf(defaultPath, sizeof(defaultPath), "%s%s%s",
-                    ((confpath == NULL) ? "" : confpath),
-                    ((perspath == NULL) ? "" : ENV_SEPARATOR),
-                    ((perspath == NULL) ? "" : perspath));
-            defaultPath[ sizeof(defaultPath)-1 ] = 0;
-	    envconfpath = strdup(defaultPath);
-        } else {
-	    envconfpath = strdup(envconfpath);
-        }
-
-        DEBUGMSGTL(("read_config", "config path used: %s\n", envconfpath));
-        cptr1 = cptr2 = envconfpath;
-        i = 1;
-        while (i && *cptr2 != 0) {
-            while (*cptr1 != 0 && *cptr1 != ENV_SEPARATOR_CHAR)
-                cptr1++;
-            if (*cptr1 == 0)
-                i = 0;
-            else
-                *cptr1 = 0;
+        perspath = get_persistent_directory();
+        if (envconfpath == NULL) {
             /*
-             * for proper persistent storage retrival, we need to read old backup
-             * copies of the previous storage files.  If the application in
-             * question has died without the proper call to snmp_clean_persistent,
-             * then we read all the configuration files we can, starting with
-             * the oldest first.
+             * read just the config files (no persistent stuff), since
+             * persistent path can change via conf file. Then get the
+             * current persistent directory, and read files there.
              */
-	    persfile = netsnmp_getenv("SNMP_PERSISTENT_FILE");
-            if (strncmp(cptr2, perspath, strlen(perspath)) == 0 ||
-                (persfile != NULL &&
-                 strncmp(cptr2, persfile, strlen(persfile)) == 0)) {
-                /*
-                 * limit this to the known storage directory only 
-                 */
-                for (j = 0; j <= MAX_PERSISTENT_BACKUPS; j++) {
-                    snprintf(configfile, sizeof(configfile),
-                           "%s/%s.%d.conf", cptr2,
-                            ctmp->fileHeader, j);
-                    configfile[ sizeof(configfile)-1 ] = 0;
-                    if (stat(configfile, &statbuf) != 0) {
-                        /*
-                         * file not there, continue 
-                         */
-                        break;
-                    } else {
-                        /*
-                         * backup exists, read it 
-                         */
-                        DEBUGMSGTL(("read_config_files",
-                                    "old config file found: %s, parsing\n",
-                                    configfile));
-                        read_config(configfile, ltmp, when);
-                    }
-                }
-            }
-            snprintf(configfile, sizeof(configfile),
-                     "%s/%s.conf", cptr2, ctmp->fileHeader);
-            configfile[ sizeof(configfile)-1 ] = 0;
-            read_config(configfile, ltmp, when);
-            snprintf(configfile, sizeof(configfile),
-                     "%s/%s.local.conf", cptr2, ctmp->fileHeader);
-            configfile[ sizeof(configfile)-1 ] = 0;
-            read_config(configfile, ltmp, when);
-            cptr2 = ++cptr1;
+            read_config_files_in_path(confpath, ctmp, when, perspath,
+                                      persfile);
+            perspath = get_persistent_directory();
+            read_config_files_in_path(perspath, ctmp, when, perspath,
+                                      persfile);
+        }
+        else {
+            /*
+             * only read path specified by user
+             */
+            read_config_files_in_path(envconfpath, ctmp, when, perspath,
+                                      persfile);
         }
-        SNMP_FREE(envconfpath);
     }
 
     if (config_errors) {
         snmp_log(LOG_ERR, "net-snmp: %d error(s) in config file(s)\n",
                  config_errors);
-        /*
-         * exit(1); 
-         */
     }
 }
 
@@ -1209,7 +1236,9 @@ read_config_store(const char *type, const char *line)
 #endif
 
     if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
-                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)) return;
+                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)
+     || netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                               NETSNMP_DS_LIB_DISABLE_PERSISTENT_LOAD)) return;
 
     /*
      * store configuration directives in the following order of preference:
@@ -1283,7 +1312,9 @@ snmp_save_persistent(const char *type)
     int             j;
 
     if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
-                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)) return;
+                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)
+     || netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                               NETSNMP_DS_LIB_DISABLE_PERSISTENT_SAVE)) return;
 
     DEBUGMSGTL(("snmp_save_persistent", "saving %s files...\n", type));
     snprintf(file, sizeof(file),
@@ -1345,7 +1376,9 @@ snmp_clean_persistent(const char *type)
     int             j;
 
     if (netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
-                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)) return;
+                               NETSNMP_DS_LIB_DONT_PERSIST_STATE)
+     || netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                               NETSNMP_DS_LIB_DISABLE_PERSISTENT_SAVE)) return;
 
     DEBUGMSGTL(("snmp_clean_persistent", "cleaning %s files...\n", type));
     snprintf(file, sizeof(file),
@@ -1630,7 +1663,13 @@ read_config_read_octet_string(char *readfrom, u_char ** str, size_t * len)
             }
             readfrom += 2;
         }
-        *cptr++ = '\0';
+        /*
+         * only null terminate if we have the space
+         */
+        if (ilen > *len) {
+            ilen = *len-1;
+            *cptr++ = '\0';
+        }
         readfrom = skip_white(readfrom);
     } else {
         /*
@@ -1820,6 +1859,7 @@ read_config_read_memory(int type, char *readfrom,
         *len = sizeof(int);
         return readfrom;
 
+    case ASN_COUNTER:
     case ASN_TIMETICKS:
     case ASN_UNSIGNED:
         if (*len < sizeof(unsigned int))
@@ -1843,6 +1883,16 @@ read_config_read_memory(int type, char *readfrom,
         *len *= sizeof(oid);
         return readfrom;
 
+    case ASN_COUNTER64:
+    {
+        if (*len < sizeof(U64))
+            return NULL;
+        *len = sizeof(U64);
+        read64((U64 *) dataptr, readfrom);
+        readfrom = skip_token(readfrom);
+        return readfrom;
+    }
+
     default:
         DEBUGMSGTL(("read_config_read_memory", "Fail: Unknown type: %d",
                     type));
@@ -1872,16 +1922,17 @@ read_config_read_memory(int type, char *readfrom,
  * @param dataptr contains the value to be stored, the supported pointers:
  *                (int *, u_int *, char **, oid **)
  *
- * @param len     is the length of the pre-allocated storeto buffer, not
- *                required for the asn integer, unsigned integer, and
- *                timeticks types.
+ * @param len     is the length of the value to be stored
+ *                (not required for the asn integer, unsigned integer,
+ *                 and timeticks types)
  *
  * @return character pointer to the end of the line. NULL if an unknown type.
  */
 char           *
 read_config_store_data(int type, char *storeto, void *dataptr, size_t * len)
 {
-    return read_config_store_data_prefix(' ', type, storeto, dataptr, *len);
+    return read_config_store_data_prefix(' ', type, storeto, dataptr,
+                                                         (len ? *len : 0));
 }
 
 char           *
