@@ -90,7 +90,7 @@ netsnmp_udp_fmtaddr(netsnmp_transport *t, void *data, int len)
     } else {
 	char tmp[64];
 
-        sprintf(tmp, "UDP: [%s]:%hd",
+        sprintf(tmp, "UDP: [%s]:%hu",
                 inet_ntoa(to->sin_addr), ntohs(to->sin_port));
         return strdup(tmp);
     }
@@ -171,6 +171,8 @@ netsnmp_udp_send(netsnmp_transport *t, void *buf, int size,
 	while (rc < 0) {
 	    rc = sendto(t->sock, buf, size, 0, to, sizeof(struct sockaddr));
 	    if (rc < 0 && errno != EINTR) {
+                DEBUGMSGTL(("netsnmp_udp", "sendto error, rc %d (errno %d)\n",
+                            rc, errno));
 		break;
 	    }
 	}
