@@ -97,6 +97,9 @@ static int      newline = 1;	 /* MTCRITICAL_RESOURCE */
 
 static char syslogname[64] = DEFAULT_LOG_ID;
 
+void
+netsnmp_enable_filelog(netsnmp_log_handler *logh, int dont_zero_log);
+
 #ifndef HAVE_VSNPRINTF
                 /*
                  * Need to use the UCD-provided one 
@@ -350,6 +353,9 @@ snmp_log_options(char *optarg, int argc, char *const *argv)
         if (logh) {
             logh->pri_max = pri_max;
             logh->token   = strdup(optarg);
+            netsnmp_enable_filelog(logh,
+                                   netsnmp_ds_get_boolean(NETSNMP_DS_LIBRARY_ID,
+                                                          NETSNMP_DS_LIB_APPEND_LOGFILES));
 	}
         break;
 
@@ -580,6 +586,33 @@ snmp_disable_log(void)
     }
 }
 
+/*
+ * close and reopen all file based logs, to allow logfile
+ * rotation.
+ */
+void
+netsnmp_logging_restart(void)
+{
+    netsnmp_log_handler *logh;
+
+    for (logh = logh_head; logh; logh = logh->next) {
+        if (0 == logh->enabled)
+            continue;
+        if (logh->type == NETSNMP_LOGHANDLER_SYSLOG) {
+            snmp_disable_syslog_entry(logh);
+            snmp_enable_syslog_ident(logh->token,(int)logh->magic);
+        }
+        else if (logh->type == NETSNMP_LOGHANDLER_FILE) {
+            snmp_disable_filelog_entry(logh);
+            /** hmm, don't zero status isn't saved.. i think it's
+             * safer not to overwrite, in case a hup is just to
+             * re-read config files...
+             */
+            netsnmp_enable_filelog(logh, 1);
+        }
+    }
+}
+
 /* ================================================== */
 
 void
@@ -672,8 +705,7 @@ snmp_enable_filelog(const char *logfilename, int dont_zero_log)
      * don't disable ALL filelogs whenever a new one is enabled.
      * this prevents '-Lf file' from working in snmpd, as the
      * call to set up /var/log/snmpd.log will disable the previous
-     * log setup. again, this new linked list of log handlers
-     * needs rethinking/cleanup. xxx-rks
+     * log setup.
      * snmp_disable_filelog();
      */
 
