@@ -105,21 +105,23 @@ GRONIK
 
 #------------------------------------ -o-
 #
+SKIP() {
+	REMOVETESTDATA
+	echo "SKIPPED"
+	exit 0
+}
+
 SKIPIFNOT() {
 	grep "^#define $1 " $SNMP_UPDIR/include/net-snmp/net-snmp-config.h $SNMP_UPDIR/include/net-snmp/agent/mib_module_config.h $SNMP_UPDIR/include/net-snmp/agent/agent_module_config.h > /dev/null
 	if [ $? != 0 ]; then
-	    REMOVETESTDATA
-	    echo "SKIPPED"
-	    exit 0;
+	    SKIP
 	fi
 }
 
 SKIPIF() {
 	grep "^#define $1 " $SNMP_UPDIR/include/net-snmp/net-snmp-config.h $SNMP_UPDIR/include/net-snmp/agent/mib_module_config.h $SNMP_UPDIR/include/net-snmp/agent/agent_module_config.h > /dev/null
 	if [ $? = 0 ]; then
-	    REMOVETESTDATA
-	    echo "SKIPPED"
-	    exit 0;
+	    SKIP
 	fi
 }
 	
@@ -253,8 +255,12 @@ CHECKTRAPD() {
     CHECKFILE $SNMP_SNMPTRAPD_LOG_FILE $@
 }
 
+CHECKTRAPDORDIE() {
+    CHECKORDIE $@ $SNMP_SNMPTRAPD_LOG_FILE
+}
+
 CHECKAGENT() {
-    CHECKGAENT $SNMP_SNMPD_LOG_FILE $@
+    CHECKFILE $SNMP_SNMPD_LOG_FILE $@
 }
 
 WAITFORAGENT() {
@@ -317,7 +323,7 @@ WAITFORORDIE() {
     ECHO "."
 }
 
-# CHECKFILE "grep string" ["file"]
+# CHECKORDIE "grep string" ["file"] .. FAIL if "grep string" is *not* found
 CHECKORDIE() {
     CHECKFILE "$2" "$1"
     if [ "$snmp_last_test_result" = 0 ] ; then
@@ -326,6 +332,16 @@ CHECKORDIE() {
     ECHO "."
 }
 
+# CHECKANDDIE "grep string" ["file"] .. FAIL if "grep string" *is* found
+CHECKANDDIE() {
+    CHECKFILE "$2" "$1"
+    EXPECTRESULT 0 # make sure return_value gets set correctly
+    if [ "$snmp_last_test_result" != 0 ] ; then
+        FINISHED
+    fi
+    ECHO "."
+}
+
 #------------------------------------ -o-
 # Returns: Count of matched lines.
 #
@@ -393,7 +409,7 @@ STARTAGENT() {
     LOG_FILE=$SNMP_SNMPD_LOG_FILE
     PORT_SPEC="$SNMP_SNMPD_PORT"
     if [ "x$SNMP_TRANSPORT_SPEC" != "x" ]; then
-        PORT_SPEC="$SNMP_TRANSPORT_SPEC:$PORT_SPEC"
+        PORT_SPEC="${SNMP_TRANSPORT_SPEC}:${SNMP_TEST_DEST}${PORT_SPEC}"
     fi
     STARTPROG
     WAITFORAGENT "NET-SNMP version"
@@ -402,17 +418,45 @@ STARTAGENT() {
 #------------------------------------ -o-
 STARTTRAPD() {
     TRAPDSTARTED=1
-    COMMAND="snmptrapd -d -u $SNMP_SNMPTRAPD_PID_FILE -Lf $SNMP_SNMPTRAPD_LOG_FILE $TRAPD_FLAGS"
+    COMMAND="snmptrapd -d -p $SNMP_SNMPTRAPD_PID_FILE -Lf $SNMP_SNMPTRAPD_LOG_FILE $TRAPD_FLAGS"
     CFG_FILE=$SNMPTRAPD_CONFIG_FILE
     LOG_FILE=$SNMP_SNMPTRAPD_LOG_FILE
     PORT_SPEC="$SNMP_SNMPTRAPD_PORT"
     if [ "x$SNMP_TRANSPORT_SPEC" != "x" ]; then
-        PORT_SPEC="$SNMP_TRANSPORT_SPEC:$PORT_SPEC"
+        PORT_SPEC="${SNMP_TRANSPORT_SPEC}:${SNMP_TEST_DEST}${PORT_SPEC}"
     fi
     STARTPROG
     WAITFORTRAPD "NET-SNMP version"
 }
 
+## sending SIGHUP for reconfiguration
+#
+HUPPROG() {
+    if [ -f $1 ]; then
+        if [ "x$OSTYPE" = "xmsys" ]; then
+          COMMAND='echo "Skipping SIGHUP (not supported by kill.exe on MinGW)"'
+        else
+          COMMAND="kill -HUP `cat $1`"
+        fi
+	echo $COMMAND >> $SNMP_TMPDIR/invoked
+	$COMMAND > /dev/null 2>&1
+    fi
+}
+
+HUPAGENT() {
+    HUPPROG $SNMP_SNMPD_PID_FILE
+    if [ "x$OSTYPE" != "xmsys" ]; then
+        WAITFORAGENT "restarted"
+    fi
+}
+
+HUPTRAPD() {
+    HUPPROG $SNMP_SNMPTRAPD_PID_FILE
+    if [ "x$OSTYPE" != "xmsys" ]; then
+        WAITFORTRAPD "restarted"
+    fi
+}
+
 
 ## used by STOPAGENT and STOPTRAPD
 # delay before kill to allow previous action to finish
